#include <fcntl.h>
#include <string.h>
#include "ui.h"
#include "ui_globals.h"
#include "sherlock.h"
#include "error.h"

int whichcrop;
char *dir[NUM_SCREENS],*file[NUM_SCREENS];
char *chdir[NUM_SCREENS],*chfile[NUM_SCREENS];
XRectangle crop_box,old_crop_box;

/*Pop up the dialog box to retrieve a directory and filename of a file to
  load or save.

  Inputs:  menu            - unused (menu that references this callback routine)
	   item            - menu item that was selected
  Outputs: none
  Locals:  none
  Globals: loadsave        - load/save dialog box UI handles
	   LOADIMAGE       - which action is to be performed (load an image)
	   LOADIMAGE_POS   - load image menu item position
	   LOADPATTERN     - which action is to be performed (load a pattern)
	   LOADPATTERN_POS - load pattern menu item position
	   MENU_VALUE      - get the position of a menu item
	   NULL            - 0
	   SAVEPATTERN     - which action is to be performed (save a pattern)
	   SAVEPATTERN_POS - save pattern menu item position
	   TRUE            - turn an attribute on
	   XV_SHOW         - map a window to the display (i.e. display window)
*/
void show_filename(menu,item)
register Menu menu;
register Menu_item item;
{
	switch(xv_get(item,MENU_VALUE)) {
	  case LOADIMAGE_POS:
			xv_set(loadsave[LOADIMAGE],XV_SHOW,TRUE,NULL);
			break;
	  case LOADPATTERN_POS:
			xv_set(loadsave[LOADPATTERN],XV_SHOW,TRUE,NULL);
			break;
	  case SAVEPATTERN_POS:
			xv_set(loadsave[SAVEPATTERN],XV_SHOW,TRUE,NULL);
			break;
	}
}

/*Quit sherlock, cleaning up the timer process and finish writing graph
  files.

  Inputs:  none
  Outputs: none
  Locals:  none
  Globals: errno - number of last error encountered
	   frame - UI window handles
	   IMAGE - reference the image element of an array
	   XV_OK - ok to destroy baseframe
*/
void quit()
{
	if(xv_destroy_safe(frame[IMAGE]) == XV_OK) {
	  errno = 0;
	  cleanup();
	}
}

/*Get text written to a browser window (the one that asks for directory and
  filename for load/save operations).

  Inputs:  event             - event that triggered this callback routine
	   item              - panel item that had the event
  Outputs: none
  Locals:  current_item      - panel item to move the caret to
	   loop              - loop through directory/filename panel items
	   value             - value in panel item
	   which	     - which window the panel item is on
  Globals: chdir             - change in directory for browser windows
	   chfile	     - change in filename for browser windows
	   errno	     - number of last error encountered
	   loadsave	     - load/save dialog box UI handles
	   DIR_HANDLE        - handle to UI directory panel item
	   FATAL	     - error is fatal, program will terminate
	   FILE_HANDLE	     - handle to UI filename panel item
	   FRAME_CMD_PANEL   - frame's command panel UI handle
	   NULL		     - 0
	   NUM_SCREENS	     - number of load/save dialog boxes
	   PANEL_CARET_ITEM  - which panel item has the caret attribute
	   PANEL_CLIENT_DATA - data attached to an xview object
	   PANEL_VALUE	     - value of UI panel item
	   XV_KEY_DATA       - data attached to an xview object
	   XV_OWNER	     - owner of an xview object
*/
Panel_setting get_browser_text(item,event)
register Event *event;
register Panel_item item;
{
	register int which;
	register char *value,loop;
	register Panel_item current_item;

	/*If the event is a carriage return or newline, get the new
	  directory or filename value.  First we have to find out which
	  load/save dialog box this event came from*/
	if((event_action(event) == '\n') || (event_action(event) == '\r')) {
	  value = (char *) xv_get(item,PANEL_VALUE);
	  for(loop = 0;loop < NUM_SCREENS;++loop)
	    if(loadsave[loop] == (Frame) xv_get(xv_get(item,XV_OWNER),
						XV_OWNER)) {
	      which = loop;
	      break;
	    }

	  /*Find out if it was a directory or filename entry*/
	  switch(xv_get(item,PANEL_CLIENT_DATA)) {
	    case 0:				/*Directory panel item*/
			/*Free space we don't need anymore*/
	  		if(chdir[which]) free(chdir[which]);

			/*If the new directory is a null string, set
			  accordingly, otherwise copy string*/
			if(value[0]) {
	  		  if((chdir[which] = strdup(value)) == NULL)
	    		    error(FATAL,errno);
			} else chdir[which] = NULL;

			/*Set filename panel item to receive caret*/
			current_item = (Panel_item) xv_get(loadsave[which],
							   XV_KEY_DATA,
							   FILE_HANDLE);

			break;
	    case 1:				/*Filename panel item*/
			/*Free space we don't need anymore*/
			if(chfile[which]) free(chfile[which]);

			/*If the new filename is a null string, set
			  accordingly, otherwise copy string*/
			if(value[0]) {
	  		  if((chfile[which] = strdup(value)) == NULL)
	    		    error(FATAL,errno);
			} else chfile[which] = NULL;

			/*Set directory panel item to receive caret*/
			current_item = (Panel_item) xv_get(loadsave[which],
							   XV_KEY_DATA,
							   DIR_HANDLE);

			break;
	  }

	  /*Advance caret to next input field*/
	  xv_set(xv_get(loadsave[which],FRAME_CMD_PANEL),PANEL_CARET_ITEM,
		 current_item,NULL);
	} else return(panel_text_notify(item,event));
}

/*Callback routine when the "Apply" or "Cancel" button is clicked on a
  browser window.  Make the window go away, copy any new information
  into the proper place, and perform the action requested (either load
  or save data).

  Inputs:  event               - event that triggered this callback routine
	   item                - panel item that had the event
  Outputs: none
  Locals:  err                 - error from readheader/writeheader (if there
				 is one)
	   fd                  - file descriptor for file to load/save
	   io_file             - file to load from/save to
	   loop                - loop through browser windows
	   opn_flg             - open() flags to open for reading or writing
	   size                - size (in characters) of full filename
				 (dir + filename)
	   which               - which browser window generated the event
	   whichimage          - which (image or pattern) is being affected
  Globals: canvas              - UI canvas handles (that display the images)
	   chdir               - change in directory for browser windows
	   chfile              - change in filename for browser windows
	   dir                 - directory set in the browser windows
	   errno               - number of last error encountered
	   file                - filename set in the browser windows
	   filename            - file to load from/save to
	   loadsave            - load/save dialog box UI handles
	   APPLY_HANDLE        - handle to UI "Apply" button
	   DIR_HANDLE          - handle to UI directory panel item
	   DISPLAY_POS         - display platten menu item position
	   ENONAME             - "no filename was specified when saving" error
	   FATAL	       - error is fatal, program will terminate
	   FILE_HANDLE         - handle to UI filename panel item
	   FILE_MENU_HANDLE    - handle to UI "File" menu button
	   LOADIMAGE           - which action is to be performed (load an image)
	   LOADPATTERN         - which action is to be performed (load a
				 pattern)
	   NULL		       - 0
	   NUM_SCREENS	       - number of load/save dialog boxes
	   O_CREAT	       - create the file if it doesn't exist
	   O_RDONLY	       - open the file for reading only
	   O_TRUNC             - truncate the file if it already exists
	   O_WRONLY            - open the file for writing only
	   PANEL_VALUE         - value of UI panel item
	   PATTERN             - reference the pattern element of an array
	   PATTERN_MENU_HANDLE - handle to UI "Pattern" menu button
	   RUN_BUTTON_HANDLE   - handle to UI "Run" menu button
	   SAVEPATTERN         - which action is to be performed (save a
				 pattern)
	   SAVEPATTERN_POS     - save pattern menu item position
	   XV_KEY_DATA         - data attached to an xview object
	   XV_OWNER            - owner of an xview object
*/
void loadsave_exit(item,event)
register Panel_item item;
register Event *event;
{
	register int fd,size,err,which,loop,whichimage,opn_flg;
	register char *io_file;

	/*Find out which browser window this event came from*/
	for(loop = 0;loop < NUM_SCREENS;++loop)
	  if(loadsave[loop] == (Frame) xv_get(xv_get(item,XV_OWNER),XV_OWNER)) {
	    which = loop;
	    break;
	  }
	whichimage = PATTERN;

	/*If the Apply button was hit, perform the requested action*/
	if(item == (Panel_item) xv_get(loadsave[which],XV_KEY_DATA,
				       APPLY_HANDLE)) {
	  /*Free up unneeded space*/
	  if(dir[which]) free(dir[which]);

	  /*If the directory was set to a null string, set accordingly,
	    otherwise copy string*/
	  if(chdir[which] &&
	     ((dir[which] = strdup(chdir[which])) == NULL))
	    error(FATAL,errno);
	  else if(!chdir[which]) dir[which] = NULL;

	  /*Free up unneeded space*/
	  if(file[which]) free(file[which]);

	  /*The filename not set is an error, if it set, process*/
	  if(chfile[which]) {
	    if((file[which] = strdup(chfile[which])) == NULL)
	      error(FATAL,errno);

	    size = strlen(file[which]);
	    if(dir[which]) size += strlen(dir[which]);
	    size += 2;

	    if((io_file = malloc(size)) == NULL) error(FATAL,errno);

	    /*Put the directory and filename together to form the full
	      filename*/
	    if(dir[which])
	      sprintf(io_file,"%s/%s",dir[which],file[which]);
	    else strcpy(io_file,file[which]);

	    /*Set up open() flags depending on loading or saving*/
	    if(which == SAVEPATTERN) opn_flg = O_WRONLY | O_CREAT | O_TRUNC;
	    else opn_flg = O_RDONLY;

	    if((fd = open(io_file,opn_flg,0644)) == -1)
	     popup_notice(errno,loadsave[which],event_x(event),event_y(event));
	    else {	/*Open() successful, carry on*/
		   switch(which) {
		     /*There is no break for LOADIMAGE, since it will
		        execute the same commands as LOADPATTERN (just with
		        a different value of "whichimage")*/
		     case LOADIMAGE:
				whichimage = IMAGE;
		     case LOADPATTERN:
				/*If we can read in a header, then resize the
				  image canvas, read the data, display the
				  data, and, in the case of a pattern, ungray
				  some menu choices*/
	           		if(!(err = readheader(fd,whichimage))) {
				  resize(whichimage);
				  readimage(fd,whichimage,NULL,(HEADER *) NULL);

				  getinfo(0,0,-1,-1);

				  /*Display data in canvas*/
				  repaint(canvas[whichimage]);

				  /*Ungray the "Save Pattern",
				    "Display Pattern", and "Run" buttons if
				    loading a pattern*/
				  if(which == LOADPATTERN) {
				    activate(FILE_MENU_HANDLE,SAVEPATTERN_POS);
				    activate(PATTERN_MENU_HANDLE,DISPLAY_POS);
				    activate(RUN_BUTTON_HANDLE,0);
				  }
				} else
				    popup_notice(err,loadsave[which],
						 event_x(event),event_y(event));

				break;
		       case SAVEPATTERN:
				/*If we can write out the header, write out
				  the data*/
				if(!(err = writeheader(fd)))
				  writeimage(fd);
				else
				    popup_notice(err,loadsave[which],
						 event_x(event),event_y(event));

				break;
		   }

		   if(filename[whichimage]) free(filename[whichimage]);
		   filename[whichimage] = io_file;
	         }
	  } else popup_notice(ENONAME,loadsave[which],event_x(event),
			      event_y(event));
	} else {	/*Cancel button was selected, change back to what
			  values were there*/
	         if(chdir[which]) free(chdir[which]);
	         if(dir[which] &&
	            ((chdir[which] = strdup(dir[which])) == NULL))
	           error(FATAL,errno);
	         else if(!dir[which]) chdir[which] = NULL;

		 /*Put the old directory string back into the browser
		   window*/
		 if(dir[which])
		   xv_set(xv_get(loadsave[which],XV_KEY_DATA,DIR_HANDLE),
			  PANEL_VALUE,dir[which],NULL);
		 else xv_set(xv_get(loadsave[which],XV_KEY_DATA,DIR_HANDLE),
			     PANEL_VALUE,"",NULL);

		 if(chfile[which]) free(chfile[which]);
		 if(file[which] &&
		    ((chfile[which] = strdup(file[which])) == NULL))
		   error(FATAL,errno);
		 else if(!file[which]) chfile[which] = NULL;

		 /*Put the old filename string back into the browser
		   window*/
		 if(file[which])
		   xv_set(xv_get(loadsave[which],XV_KEY_DATA,FILE_HANDLE),
			  PANEL_VALUE,file[which],NULL);
		 else xv_set(xv_get(loadsave[which],XV_KEY_DATA,FILE_HANDLE),
			     PANEL_VALUE,"",NULL);
	       }
}

/*Crop an area of the image to be the pattern (in UI mode).

  Inputs:  none
  Outputs: none
  Locals:  drawable            - X Window drawable of the image to crop from
  Globals: canvas              - UI canvas handles (that display the images)
	   crop_box            - crop area coordinates
	   filename            - image/pattern filename
	   old_crop_box        - old crop area coordinates
	   whichcrop           - which canvas has the area to crop
	   DISPLAY_POS         - display platten menu item position
	   FILE_MENU_HANDLE    - handle to UI "File" menu button
	   NULL		       - 0
	   PATTERN             - reference the pattern element of an array
	   PATTERN_MENU_HANDLE - handle to UI "Pattern" menu button
	   RUN_BUTTON_HANDLE   - handle to UI "Run" menu button
	   SAVEPATTERN_POS     - save pattern menu item position
	   SELECT_POS          - select pattern menu item position
	   WIN_CLIENT_DATA     - data attached to an xview object
*/
void select_pattern()
{
	register Drawable drawable;

	/*Erase the old crop box*/
	drawable = (Drawable) xv_get(canvas[whichcrop],WIN_CLIENT_DATA);
	drawbox(drawable,old_crop_box);

	/*Activate the "save pattern", "display pattern", and "Run" menu
	  buttons and deactivate the "Select pattern" menu button*/
	activate(FILE_MENU_HANDLE,SAVEPATTERN_POS);
	deactivate(PATTERN_MENU_HANDLE,SELECT_POS);
	activate(PATTERN_MENU_HANDLE,DISPLAY_POS);
	activate(RUN_BUTTON_HANDLE,0);

	if(filename[PATTERN]) free(filename[PATTERN]);
	filename[PATTERN] = NULL;

	/*Get crop coordinates and cropped data*/
	getinfo(crop_box.x,crop_box.y,crop_box.width,crop_box.height);
	getimage(whichcrop,crop_box);

	crop_box.x = -1;
}

/*Display a pattern in a canvas.

  Inputs:  none
  Outputs: none
  Locals:  rect         - location of mouse
  Globals: canvas       - UI canvas handles (that display the images)
	   frame        - UI window handles
	   image        - array containing image and pattern data
	   ENOPATTERN   - "no pattern in memory to display" error
	   FALSE        - turn attribute off
	   FRAME_CLOSED - frame is an icon attribute
	   NULL         - 0
	   PATTERN      - reference the pattern element of an array
	   WIN_MOUSE_XY - mouse coordinates
*/
void display_pattern()
{
	register Rect *rect;

	/*If a pattern exists, display it, otherwise popup an error message*/
	if(image[PATTERN]) {
	  xv_set(frame[PATTERN],FRAME_CLOSED,FALSE,NULL);
	  repaint(canvas[PATTERN]);
	} else {
	         rect = (Rect *) xv_get(frame[IMAGE],WIN_MOUSE_XY);
	         popup_notice(ENOPATTERN,frame[IMAGE],rect->r_left,rect->r_top);
	       }
}

/*Track mouse events on a canvas so that a cropping box can be displayed
  (controlled by dragging the mouse).

  Inputs:  event               - event that triggered this callback routine
	   window              - window this event came from
  Outputs: none
  Locals:  changed             - flag indicating the box has changed size at
				 least once
	   draw                - flag indicating if the box should be drawn
	   drawable            - X Window drawable to draw box to
	   mouse_pos           - user readable mouse coordinate message
  Globals: canvas              - UI canvas handles (that display the images)
	   crop_box            - crop area coordinates
	   frame               - UI window handles
	   old_crop_box        - old crop area coordinates
	   whichcrop           - which canvas has the area to crop
	   ACTION_SELECT       - select button was pressed (usually the left
				 button)
	   FRAME_RIGHT_FOOTER  - window right footer string attribute
	   IMAGE               - reference the image element of an array
	   LOC_DRAG            - mouse is being dragged (while button down)
				 event
	   LOC_MOVE	       - mouse is being moved event
	   NULL		       - 0
	   PATTERN	       - reference the pattern element of an array
	   PATTERN_MENU_HANDLE - handle to UI "Pattern" menu button
	   SELECT_POS          - select pattern menu item position
	   WIN_CLIENT_DATA     - data attached to an xview object
*/
void mouse_event(window,event)
register Event *event;
register Xv_Window window;
{
	register int draw = 0;
	static int changed = 0;
	register char mouse_pos[12];
	register Drawable drawable;

	/*Find out which canvas the event came from*/
	if(window == canvas_paint_window(canvas[IMAGE])) whichcrop = IMAGE;
	else whichcrop = PATTERN;

	/*Do action associated with a particular event*/
	switch(event_action(event)) {
	  case LOC_MOVE:			/*Mouse moved across canvas*/
		/*Display mouse coordinates to the user*/
		sprintf(mouse_pos,"[%d,%d]",event_x(event),event_y(event));
		xv_set(frame[IMAGE],FRAME_RIGHT_FOOTER,mouse_pos,NULL);

		break;
	  case LOC_DRAG:			/*Mouse dragged across canvas*/
		/*Get new coordinates of cropping box*/
		old_crop_box = crop_box;

		crop_box.width = event_x(event) - crop_box.x;
		crop_box.height = event_y(event) - crop_box.y;

		draw = 1;
		changed = 1;

		break;
	  case ACTION_SELECT:			/*Select mouse button selected*/
		/*If the event was that the select mouse button was released,
		  this means that the user has finised changing the size of
		  the crop box*/
		if(event_is_up(event))  {
		  if(!changed) old_crop_box.x = -1;
		  else old_crop_box = crop_box;

		  activate(PATTERN_MENU_HANDLE,SELECT_POS);
		} else {
			 /*The select button was pressed down meaning the
			   start of drawing the crop box*/
			 old_crop_box = crop_box;

			 crop_box.x = event_x(event);
			 crop_box.y = event_y(event);
			 crop_box.width = 0;
			 crop_box.height = 0;

			 changed = 0;
		       }

		draw = 1;

		break;
	}

	/*If the event had to do with the crop box, redraw it*/
	if(draw) {
	  drawable = (Drawable) xv_get(canvas[whichcrop],WIN_CLIENT_DATA);

	  if(old_crop_box.x != -1) drawbox(drawable,old_crop_box);
	  drawbox(drawable,crop_box);
	}
}

/*Get the algorithm menu choice the user has selected and run that
  algorithm.

  Inputs:  item       - menu item that was selected
	   menu       - unused (menu that references this callback routine)
  Outputs: none
  Locals:  none
  Globals: dpy        - X display
	   frame      - UI window handles
	   FRAME_BUSY - set to indicate a window is busy (i.e. hourglass cursor)
	   GAUGE      - reference the xview gauge object of an array
	   IMAGE      - reference the image element of an array
	   MENU_VALUE - get the position of a menu item
	   NULL       - 0
	   PATTERN    - reference the pattern element of an array
	   TRUE       - turn an attribute on
	   XV_SHOW    - map a window to the display (i.e. display window)
*/
void get_alg(menu,item)
register Menu menu;
register Menu_item item;
{
	/*Display the gauge and set to 0% completed*/
	update_gauge(0.0);
	xv_set(frame[GAUGE],XV_SHOW,TRUE,NULL);

	/*Set footers in the windows to indicate to the user what's
	  happening*/
        set_footer("Searching for pattern...",IMAGE);
        set_footer("Searching for pattern...",PATTERN);

	/*Set windows to their busy state so the user can't mess anything
	  up*/
        xv_set(frame[IMAGE],FRAME_BUSY,TRUE,NULL);
        xv_set(frame[PATTERN],FRAME_BUSY,TRUE,NULL);

	/*Since #defines for the different algorithms start at 0 (like any
	  good C index should), subtracting 1 from the menu item position
	  will make a match*/
	run((int) xv_get(item,MENU_VALUE) - 1);

	/*Unmap the gauge*/
	xv_set(frame[GAUGE],XV_SHOW,FALSE,NULL);

	/*Release the windows from the busy state*/
        xv_set(frame[IMAGE],FRAME_BUSY,FALSE,NULL);
        xv_set(frame[PATTERN],FRAME_BUSY,FALSE,NULL);

	/*Indicate to the user Sherlock is ready to continue*/
        set_footer("Ready",IMAGE);
        set_footer("Ready",PATTERN);

	/*Sound the bell to let the user know Sherlock is done*/
        XBell(dpy,50);
        XBell(dpy,50);

	/*Display the results of the algorithm*/
	display_results();
}

