#include "sherlock.h"

extern double maxcorr,*corrmatrix;

/*Calculate the correlation between to matrices (image and pattern).  Instead
  of the normal correlation of adding up the pixel-by-pixel multiplies, we
  calculate the absolute value of the difference between corresponding pixels.
  We then look for a minimum to find the greatest correlation (i.e. the least
  difference between matrices).  This speeds up the processing by not having
  to 1) calculate the average for pattern and image and 2) subtracting that
  average from each pixel.  Also, by not having to subtract the average, we
  can do the subtraction with integers, not floating point numbers.

  Inputs:  position   - position to store value in corrmatrix
	   xpos       - X position to start matrix within the image
	   ypos       - Y position to start matrix within the image
  Outputs: result     - correlation result
  Locals:  dindex     - index to pattern data
	   loopx      - loop through matrix pixels to correlate
	   loopy      - loop through matrix scanlines to correlate
	   result     - correlation result
	   value      - image data value (which is 0 if we are accessing
			"virtual" pixels outside the physical image)
	   xend       - last pixel within the image to use
	   xstart     - first pixel within the image to use
	   yend       - last scanline within the image to use
	   ystart     - first scanline within the image to use
  Globals: corrmatrix - matrix to store correlation values
	   data       - image/pattern data arrays
	   header     - image/pattern header information arrays
	   maxcorr    - maximum correlation value
	   IMAGE      - reference the image element of an array
	   PATTERN    - reference the pattern element of an array
*/
double corr(xpos,ypos,position)
register long xpos,ypos,position;
{
	register long loopx,loopy,dindex,value,xstart,xend,ystart,yend;
	register double result;

	/*Figure out the starting and ending pixels and scanlines within
	  the image to correlate the pattern against*/
	xstart = xpos - (header[PATTERN].pixels/2);
	ystart = ypos - (header[PATTERN].scanlines/2);
	xend = xstart + header[PATTERN].pixels;
	yend = ystart + header[PATTERN].scanlines;

	result = 0.0;

	/*Go through pixel-by-pixel, adding up the differences between
	  pixels*/
	for(loopy = ystart,dindex = 0;loopy < yend;++loopy)
	  for(loopx = xstart;loopx < xend;++loopx) {
            if((loopx >= 0) && (loopx < header[IMAGE].pixels) && 
               (loopy >= 0) && (loopy < header[IMAGE].scanlines))
	      value = (long) data[IMAGE][cvt(loopx,loopy,IMAGE)];
	    else
	      /*Get background value*/
	      value = (long) data[IMAGE][cvt(0,0,IMAGE)];

	    result += (double) abs(value - (long) data[PATTERN][dindex++]);
	  }

	/*Store correlation value*/
	corrmatrix[position] = result;
	if(result > maxcorr)
	  maxcorr = result;

	return(result);
}

