/*     Fourtx
 *    Cooley-Tukey Fast Fourier Transform in C.
 *    Multi-dimensional transform, dimensions of arbitrary size,
 *    complex or real data.  N points can be transformed in time
 *    proportional to n*log(n), whereas other methods take n**2 time.
 *    Furthermore, less error is built up.  Written in USASI BASIC FORTRAN
 *    by Norman Brenner of MIT Lincoln Laboratory, June 1968.
 *
 *    Translated to C by Keith Knox of Xerox Webster Research Center,
 *    September 1990.  Xerox Corp. makes no representations about the
 *    suitability of this algorithm or software for any purpose.  It is
 *    provided "as is" without express or implied warranty.
 *
 *    Dimension data(n(1),n(2),...),transform(n(1),n(2),...),n(ndim)
 *    Transform(k1,k2,...) = sum(data(j1,j2,...)*exp(isign*2*pi*sqrt(-1)
 *    *((j1-1)*(k1-1)/n(1)+(j2-1)*(k2-1)/n(2)+...))), summed for all
 *    j1 and k1 from 1 to n(1), j2 and k2 from 1 to n(2), etc. for all
 *    ndim subscripts.  Ndim must be positive and each n(idim) may be
 *    any integer.  Isign is +1 or -1.  Let ntot = n(1)*n(2)...
 *    ...*n(ndim).  Then a -1 transform followed by a +1 one
 *    (or vice versa) returns ntot times the original data.
 *    Iform = 1, 0 or -1, as data is complex, real or the
 *    first half of a complex array.  Transform values are
 *    returned to array data.  They are complex, real or
 *    the first half of a complex array, as iform = 1, -1 or 0.
 *    The transform of a real array (iform = 0) dimensioned n(1) by n(2)
 *    by ... will be returned in the same array, now considered to
 *    be complex of dimensions n(1)/2+1 by n(2) by ....  Note that if
 *    iform = 0 or -1, n(1) must be even, and enough room must be
 *    reserved.  The missing values may be obtained by complex conju-
 *    gation.  The reverse transformation, of a half complex array
 *    dimensioned n(1)/2+1 by n(2) by ..., is accomplished setting iform
 *    to -1.  In the n array, n(1) must be the true n(1), not n(1)/2+1.
 *    The transform will be real and returned to the input array.
 *    Work is a one-dimensional complex array used for working storage.
 *    Its length, nwork, need never be larger than the largest n(idim)
 *    and frequently may be much smaller.  Fourt computes the minimum
 *    length working storage required and checks that nwork is at least
 *    as long.  This minimum length is computed as shown below.
 *
 *    For example--
 *    dimension data(1960),work(10)
 *    complex data,work
 *    fourt(data, 1960, 1, -1, +1, work, 10)
 *
 *    The multi-dimensional transform is broken down into one-dimen-
 *    sional transforms of length n(idim).  These are further broken
 *    down into transforms of length ifact(if), where these are the
 *    prime factors of n(idim).  For example, n(1) = 1960, ifact(if) =
 *    2, 2, 2, 5, 7 and 7.  The running time is proportional to ntot *
 *    sum(ifact(if)), though factors of two and three will run espe-
 *    cially fast.  Naive transform programs will run in time ntot**2.
 *    Arrays whose size ntot is prime will run much slower than those
 *    with composite ntot.  For example, ntot = n(1) = 1951 (a prime),
 *    running time will be 1951*1951, while for ntot = 1960, it will
 *    be 1960*(2+2+2+5+7+7), a speedup of eighty times.  Naive calcul-
 *    ation will run both in the slower time.  If an array is of
 *    inconvenient length, simply add zeroes to pad it out.  The results
 *    will be interpolated according to the new length (see below).
 *
 *    A Fourier transform of length ifact(if) requires a work array
 *    of that length.  Therefore, nwork must be as big as the largest
 *    prime factor.  Further, work is needed for digit reversal--
 *    each n(idim) (but n(1)/2 if iform = 0 or -1) is factored symmetri-
 *    cally, and nwork must be as big as the center factor.  (To factor
 *    symmetrically, separate pairs of identical factors to the flanks,
 *    combining all leftovers in the center.)  For example, n(1) = 1960
 *    =2*2*2*5*7*7=2*7*10*7*2, so nwork must at least max(7,10) = 10.
 *
 *    An upper bound for the rms relative error is given by Gentleman
 *    and Sande (3)-- 3 * 2**(-b) * sum(f**1.5), where 2**(-b) is the
 *    smallest bit in the floating point fraction and the sum is over
 *    the prime factors of ntot.
 *
 *    If the input data are a time series, with index j representing
 *    a time (j-1)*deltat, then the corresponding index k in the
 *    transform represents the frequency (k-1)*2*pi/(n*deltat), which
 *    by periodicity, is the same as frequency -(n-k+1)*2*pi/(n*deltat).
 *    This is true for n = each n(idim) independently.
 *
 *    References--
 *    1.  Cooley, J.W. and Tukey, J.W., An Algorithm for the Machine
 *    Calculation of Complex Fourier Series.  Math. Comp., 19, 90,
 *    (April 1967), 297-301.
 *    2.  Rader, C., et al., What is the Fast Fourier Transform, IEEE
 *    Transactions on Audio and Electroacoustics, AU-15, 2 (June 1967).
 *    (Special Issue on the Fast Fourier Transform and Its Applications)
 *    3.  Gentleman, W.M. and Sande, G., Fast Fourier Transforms--
 *    for Fun and Profit.  1966 Fall Joint Comp. Conf., Spartan Books,
 *    Washington, 1966.
 *    4.  Goertzel, G., An Algorithm for the Evaluation of Finite
 *    Trigonometric Series.  Am. Math. Mo., 65, (1958), 34-35.
 *    5.  Singleton, R.C., A Method for Computing the Fast Fourier
 *    Transform with Auxiliary Memory and Limited High-speed Storage.
 *    in (2).
 */


#include <stdio.h>
#include <math.h>

#define err0 "fourt: Iform=%d, but n[0] is not even. Dimensions=%s.\n"
#define err1 "fourt: Nwork=%d is too small for n[%d]=%d, whose center=%d and whose prime factors are %s.\n"

/* Arguments into fourt() are by value, in accordance with C conventions. */

fourt(data, n, ndim, isign, iform, work, nwork)
  double *data;
  int *n, ndim, isign, iform;
  double *work;
  int nwork;
  {
  /* Decrement all of the arrays by 1 to convert them to FORTRAN addresses. */
  int ifsym_[32], ifcnt_[10], ifact_[32];
  int *ifsym, *ifcnt, *ifact;
  int idim, jdim, ntot, nrem, ncurr, nfact;
  int isym, nfsym, icent, nfcnt, ifmax, nprev;
  int temp1, temp2;
  char string[200];
  ifsym = ifsym_-1;
  ifcnt = ifcnt_-1;
  ifact = ifact_-1;
  data--;
  n--;
  work--;
  if (iform <= 0 && (n[1]-2*(n[1]/2)) != 0)
    {
    errorstring(string, n, ndim);
    fprintf(stderr, err0, iform, string); 
    return;
    }
  ntot = 1;
  for (idim=1; idim <= ndim; idim++) ntot *= n[idim];
  nrem = ntot;
  if (iform < 0)
    {
    nrem = 1;
    ntot = (ntot/n[1])*(n[1]/2+1);
    }
  /* Loop over all dimensions. */
  for (jdim=1; jdim <= ndim; jdim++)
    {
    if (iform < 0) idim = ndim+1-jdim;
    else
      {
      idim = jdim;
      nrem /= n[idim];
      }
    ncurr = n[idim];
    if (idim <= 1)
      {
      if (iform < 0)
        {
        fixrl(data, n+1, &nrem, &isign, &iform);
        ntot = (ntot/(n[1]/2+1))*n[1];
        }
      if (iform <= 0) ncurr /= 2;
      }
    if (ncurr > 1)
      {
      /* Factor n[idim], the length of this dimension. */
      factr(&ncurr, ifact, &nfact);
      ifmax = ifact[nfact];
      /* Arrange the factors symmetrically for simpler digit reversal. */
      smfac(ifact, &nfact, &isym, ifsym, &nfsym, &icent, ifcnt, &nfcnt);
      ifmax = (ifmax > icent) ? ifmax : icent;
      if (ifmax > nwork)
        {
        errorstring(string, ifact, nfact);
        fprintf(stderr, err1, nwork, idim, ncurr, icent, string);
        return;
        }
      nprev = ntot/(n[idim]*nrem);
      /* Digit reverse on symmetric factors, for example 2*7*6*7*2. */
      symrv(data, &nprev, &ncurr, &nrem, ifsym, &nfsym);
      /* Digit reverse the asymmetric center, for example, on 6 = 2*3. */
      temp1 = nprev*isym;
      temp2 = isym*nrem;
      asmrv(data, &temp1, &icent, &temp2, ifcnt, &nfcnt, work);
      /* Fourier transform on each factor, for example, on 2,7,2,3,7 and 2. */
      cool(data, &nprev, &ncurr, &nrem, &isign, ifact, work);
      }
    if (iform < 0) nrem *= n[idim];
    else
      {
      if (iform == 0 && idim <= 1)
        {
        fixrl(data, n+1, &nrem, &isign, &iform);
        ntot = (ntot/n[1])*(n[1]/2+1);
        }
      }
    }
  }



/***************************************************************************
 * factrx
 *
 * Factor n into its prime factors, nfact in number.  For example,
 *    for n = 1960, nfact = 6 and ifact(if) = 2, 2, 2, 5, 7 and 7.
 *
***************************************************************************/

factr(n, ifact, nfact)
  int *n, *ifact, *nfact;
  {
  int if_, npart, id, idiv, iquot;
  if_ = 0;
  npart = *n;
  for (id=1; id <= *n; id += 2)
    {
    idiv = id;
    if (id <= 1) idiv = 2;
    while (npart == idiv*(iquot = npart/idiv))
      {
      if_++;
      ifact[if_] = idiv;
      npart = iquot;
      }
    if (iquot <= idiv) break;
    }
  if (npart > 1)
    {
    if_++;
    ifact[if_] = npart;
    }
  *nfact = if_;
  }



/***************************************************************************
 * smfacx
 *
 * Rearrange the prime factors of n into a square and a non-square.
 *   n = isym*icent*isym, where icent is square-free.
 *   isym = ifsym[1]*...*ifsym[nfsym], each a prime factor.
 *   icent = ifcnt[1]*...*ifcnt[nfcnt], each a prime factor.
 *
 * For example, n = 1960 = 14*10*14.  Then isym = 14, icent = 10,
 *   nfsym = 2, nfcnt = 2, nfact = 6, ifsym[ifs] = 2, 7,
 *   ifcnt[ifc] = 2, 5 and ifact[if] = 2, 7, 2, 5, 7, 2.
 *
***************************************************************************/

smfac(ifact, nfact, isym, ifsym, nfsym, icent, ifcnt, nfcnt)
  int *ifact, *nfact, *isym, *ifsym, *nfsym, *icent, *ifcnt, *nfcnt;
  {
  int ifs, ifc, if_, nfsm2, ifscj, ifcnj;
  *isym = 1;
  *icent = 1;
  ifs = 0;
  ifc = 0;
  if_ = 1;
  while (if_ <= *nfact)
    {
    if (if_ < *nfact)
      {
      if (ifact[if_] == ifact[if_+1])
        {
        ifs++;
        ifsym[ifs] = ifact[if_];
        *isym = ifact[if_]*(*isym);
        if_ += 2;
        continue;
        }
      }
    ifc++;
    ifcnt[ifc] = ifact[if_];
    *icent = ifact[if_]*(*icent);
    if_++;
    }
  *nfsym = ifs;
  *nfcnt = ifc;
  nfsm2 = 2*(*nfsym);
  *nfact = nfsm2+(*nfcnt);
  if (*nfcnt > 0)
    {
    nfsm2++;
    ifsym[(*nfsym)+1] = *icent;
    for (ifc=1; ifc <= *nfcnt; ifc++)
      {
      if_ = *nfsym+ifc;
      ifact[if_] = ifcnt[ifc];
      }
    }
  if (*nfsym > 0)
    {
    for (ifs=1; ifs <= *nfsym; ifs++)
      {
      ifscj = nfsm2+1-ifs;
      ifsym[ifscj] = ifsym[ifs];
      ifact[ifs] = ifsym[ifs];
      ifcnj = (*nfact)+1-ifs;
      ifact[ifcnj] = ifsym[ifs];
      }
    }
  *nfsym = nfsm2;
  }



/***************************************************************************
 * symrvx
 *
 * Shuffle the data array by reversing the digits of one index.
 *
 *     dimension data(nprev,n,nrem)
 *
 * Replace data(i1,i2,i3) by data(i1,i2rev,i3) for all i1 from 1 to
 * nprev, i2 from 1 to n and i3 from 1 to nrem.  i2rev-1 is the
 * integer whose digit representation in the multi-radix notation
 * of factors ifact(if) is the reverse of the representation of i2-1.
 *
 * For example, if all ifact(if) = 2, i2-1 = 11001, i2rev-1 = 10011.
 * the factors must be symmetrically arranged, i.e., ifact(if) =
 * ifact(nfact+1-if).
 *
***************************************************************************/
symrv(data, nprev, n, nrem, ifact, nfact)
  double *data;
  int *nprev, *n, *nrem, *ifact, *nfact;
  {
  int ip0, ip1, ip2, ip3, ip4, ip5;
  int i1, i4, i5, i4rev, i5rev, i1max, if_;
  double tempr, tempi;
  if (*nfact <= 1) return;
  ip0 = 2;
  ip1 = ip0*(*nprev);
  ip4 = ip1*(*n);
  ip5 = ip4*(*nrem);
  i4rev = 1;
  for (i4=1; i4 <= ip4; i4 += ip1)
    {
    if (i4 < i4rev)
      {
      i1max = i4+ip1-ip0;
      for (i1=i4; i1 <= i1max; i1 += ip0)
        {
        for (i5=i1; i5 <= ip5; i5 += ip4)
          {
          i5rev = i4rev+i5-i4;
          tempr = data[i5];
          tempi = data[i5+1];
          data[i5] = data[i5rev];
          data[i5+1] = data[i5rev+1];
          data[i5rev] = tempr;
          data[i5rev+1] = tempi;
          }
        }
      }
    ip3 = ip4;
    for (if_=1; if_ <= *nfact; if_++)
      {
      ip2 = ip3/ifact[if_];
      i4rev += ip2;
      if (i4rev <= ip3) break;
      i4rev -= ip3;
      ip3 = ip2;
      }
    }
  }





/***************************************************************************
 * asmrvx
 *
 * Shuffle the data array by reversing the digits of one index.
 * The operation is the same as in symrv, except that the factors
 * need not be symmetrically arranged, i.e., generally ifact[if_] not=
 * ifact[nfact+1-if_].  Consequently, a work array of length n is needed.
 *
***************************************************************************/
asmrv(data, nprev, n, nrem, ifact, nfact, work)
  double *data;
  int *nprev, *n, *nrem, *ifact, *nfact;
  double *work;
  {
  int ip0, ip1, ip2, ip3, ip4, ip5;
  int i1, i4, i5, i4rev, i4max, iwork, if_;
  if (*nfact <= 1) return;
  ip0 = 2;
  ip1 = ip0*(*nprev);
  ip4 = ip1*(*n);
  ip5 = ip4*(*nrem);
  for (i1=1; i1 <= ip1; i1 +=ip0)
    {
    for (i5=i1; i5 <= ip5; i5 += ip4)
      {
      iwork = 1;
      i4rev = i5;
      i4max = i5+ip4-ip1;
      for (i4=i5; i4 <= i4max; i4 += ip1)
        {
        work[iwork] = data[i4rev];
        work[iwork+1] = data[i4rev+1];
        ip3 = ip4;
        for (if_=1; if_ <= *nfact; if_++)
          {
          ip2 = ip3/ifact[if_];
          i4rev = i4rev+ip2;
          if (i4rev < (ip3+i5)) break;
          i4rev = i4rev-ip3;
          ip3 = ip2;
          }
        iwork = iwork+ip0;
        }
      iwork = 1;
      for (i4=i5; i4 <= i4max; i4 += ip1)
        {
        data[i4] = work[iwork];
        data[i4+1] = work[iwork+1];
        iwork = iwork+ip0;
        }
      }
    }
  }



/***************************************************************************
 * coolx
 *
 * cool(data, nprev, n, nrem, isign, ifact, work)
 *
 * Fourier transform of length n.  In place Cooley-Tukey method,
 * digit-reversed to normal order, Sande-Tukey factoring (2).
 *
 *     dimension data(nprev,n,nrem)
 *     complex data
 *
 * data[i1,j2,i3] = sum(data[i1,i2,i3]*exp(isign*2*pi*i*((i2-1)*
 * (j2-1)/n))), summed over i2 = 1 to n for all i1 from 1 to nprev,
 * j2 from 1 to n and i3 from 1 to nrem.  The factors of n are given
 * in any order in array ifact.  Factors of two are done in pairs
 * as much as possible (Fourier transform of length four), factors of
 * three are done separately, and all factors five or higher
 * are done by Goertzel's algorithm (4).
 *
***************************************************************************/
cool(data, nprev, n, nrem, isign, ifact, work)
  double *data;
  int *nprev, *n, *nrem, *isign, *ifact;
  double *work;
  {
  int ip0, ip1, ip2, ip3, ip4, ip5, if_, ifcur, temp, temp1, temp2;
  int j0, j1, j2, j3, i1, i2, i5, i1max;
  double twopi, theta, sinth, rootr, rooti;
  double wr, wi, w2r, w2i, w3r, w3i, wstpr, wstpi;
  double tempr, tempi, sumr, sumi, difr, difi;
  double t0r, t0i, t1r, t1i, t2r, t2i, t3r, t3i;
  twopi = 6.283185307*((double) (*isign));
  ip0 = 2;
  ip1 = ip0*(*nprev);
  ip4 = ip1*(*n);
  ip5 = ip4*(*nrem);
  if_ = 0;
  ip2 = ip1;
  while (ip2 < ip4)
    {
    if_++;
    ifcur = ifact[if_];
    if ((ifcur == 2) && (4*ip2 <= ip4) && (ifact[if_+1] == 2))
      {
      if_++;
      ifcur = 4;
      }
    ip3 = ip2*ifcur;
    theta = twopi/((double) ifcur);
    sinth = sin(theta/2.);
    rootr = -2.*sinth*sinth;
    /* cos(theta)-1, for accuracy. */
    rooti = sin(theta);
    theta = twopi/((double) (ip3/ip1));
    sinth = sin(theta/2.);
    wstpr = -2.*sinth*sinth;
    wstpi = sin(theta);
    wr = 1.;
    wi = 0.;
    for (i2=1; i2 <= ip2; i2 += ip1)
      {
      if (ifcur > 4)
        {
        /* Do a Fourier transform of length five or more. */
        temp1 = ip2/ip1;
        temp2 = ip5/ip3;
        goert(data+i2-1, nprev, &temp1, &ifcur, &temp2, work, &wr, &wi, &rootr, &rooti);
        }
      else
        {
        temp = (i2-1)*(ifcur-2);
        if (temp < 0) return;
        if (temp > 0)
          {
          w2r = wr*wr-wi*wi;
          w2i = 2.*wr*wi;
          w3r = w2r*wr-w2i*wi;
          w3i = w2r*wi+w2i*wr;
          }
        i1max = i2+ip1-ip0;
        for (i1=i2; i1 <= i1max; i1 += ip0)
          {
          for (i5=i1; i5 <= ip5; i5 += ip3)
            {
            j0 = i5;
            j1 = j0+ip2;
            j2 = j1+ip2;
            j3 = j2+ip2;
            if (i2 > 1)
              {
              if (ifcur > 3)
                {
                /* Apply the phase shift factors. */
                tempr = data[j3];
                data[j3] = w3r*tempr-w3i*data[j3+1];
                data[j3+1] = w3r*data[j3+1]+w3i*tempr;
                tempr = data[j2];
                data[j2] = wr*tempr-wi*data[j2+1];
                data[j2+1] = wr*data[j2+1]+wi*tempr;
                tempr = data[j1];
                data[j1] = w2r*tempr-w2i*data[j1+1];
                data[j1+1] = w2r*data[j1+1]+w2i*tempr;
                }
              else
                {
                if (ifcur == 3)
                  {
                  tempr = data[j2];
                  data[j2] = w2r*tempr-w2i*data[j2+1];
                  data[j2+1] = w2r*data[j2+1]+w2i*tempr;
                  }
                tempr = data[j1];
                data[j1] = wr*tempr-wi*data[j1+1];
                data[j1+1] = wr*data[j1+1]+wi*tempr;
                }
              }
            if (ifcur < 3)
              {
              /* Do a fourier transform of length two. */
              tempr = data[j1];
              tempi = data[j1+1];
              data[j1] = data[j0]-tempr;
              data[j1+1] = data[j0+1]-tempi;
              data[j0] = data[j0]+tempr;
              data[j0+1] = data[j0+1]+tempi;
              continue;
              }
            if (ifcur == 3)
              {
              /* Do a fourier transform of length three. */
              sumr = data[j1]+data[j2];
              sumi = data[j1+1]+data[j2+1];
              tempr = data[j0]-.5*sumr;
              tempi = data[j0+1]-.5*sumi;
              data[j0] = data[j0]+sumr;
              data[j0+1] = data[j0+1]+sumi;
              difr = rooti*(data[j2+1]-data[j1+1]);
              difi = rooti*(data[j1]-data[j2]);
              data[j1] = tempr+difr;
              data[j1+1] = tempi+difi;
              data[j2] = tempr-difr;
              data[j2+1] = tempi-difi;
              continue;
              }
            /* Do a fourier transform of length four (from bit reversed order). */
            t0r = data[j0]+data[j1];
            t0i = data[j0+1]+data[j1+1];
            t1r = data[j0]-data[j1];
            t1i = data[j0+1]-data[j1+1];
            t2r = data[j2]+data[j3];
            t2i = data[j2+1]+data[j3+1];
            t3r = data[j2]-data[j3];
            t3i = data[j2+1]-data[j3+1];
            data[j0] = t0r+t2r;
            data[j0+1] = t0i+t2i;
            data[j2] = t0r-t2r;
            data[j2+1] = t0i-t2i;
            if (*isign <= 0)
              {
              t3r = -t3r;
              t3i = -t3i;
              }
            data[j1] = t1r-t3i;
            data[j1+1] = t1i+t3r;
            data[j3] = t1r+t3i;
            data[j3+1] = t1i-t3r;
            }
          }
        }
      tempr = wr;
      wr = wstpr*tempr-wstpi*wi+tempr;
      wi = wstpr*wi+wstpi*tempr+wi;
      }
    ip2 = ip3;
    }
  }



/***************************************************************************
 * goertx
 *
 * Phase-shifted Fourier transform of length ifact by the Goertzel
 * algorithm (4).  ifact must be odd and at least 5.  Further speed
 * is gained by computing two transform values at the same time.
 *
 * dimension data(nprev,iprod,ifact,irem)
 *
 * data(i1,1,j3,i5) = sum(data(i1,1,i3,i5) * w**(i3-1)), summed
 * over i3 = 1 to ifact for all i1 from 1 to nprev, j3 from 1 to
 * ifact and i5 from 1 to irem.
 * w = wmin * exp(isign*2*pi*i*(j3-1)/ifact).
 *
***************************************************************************/
goert(data, nprev, iprod, ifact, irem, work, wminr, wmini, rootr, rooti)
  double *data;
  int *nprev, *iprod, *ifact, *irem;
  double *work;
  double  *wminr, *wmini, *rootr, *rooti;
  {
  int ip0, ip1, ip2, ip3, ip5, i1max, i3max, i3min;
  int i1, i3, i5, iwmin, iwmax, iwork, iwcnj, doit;
  double wr, wi, tempr, tempi, sumr, sumi, twowr, oldsr, oldsi;
  ip0 = 2;
  ip1 = ip0*(*nprev);
  ip2 = ip1*(*iprod);
  ip3 = ip2*(*ifact);
  ip5 = ip3*(*irem);
  if (*wmini != 0.0)
    {
    /* Apply the phase shift factors. */
    wr = *wminr;
    wi = *wmini;
    i3min = 1+ip2;
    for (i3=i3min; i3 <= ip3; i3 += ip2)
      {
      i1max = i3+ip1-ip0;
      for (i1=i3; i1 <= i1max; i1 += ip0)
        {
        for (i5=i1; i5 <= ip5; i5 += ip3)
          {
          tempr = data[i5];
          data[i5] = wr*tempr-wi*data[i5+1];
          data[i5+1] = wr*data[i5+1]+wi*tempr;
          }
        }
      tempr = wr;
      wr = (*wminr)*tempr-(*wmini)*wi;
      wi = (*wminr)*wi+(*wmini)*tempr;
      }
    }
  for (i1=1; i1 <= ip1; i1 += ip0)
    {
    for (i5=i1; i5 <= ip5; i5 += ip3)
      {
      /* Straight summation for the first term. */
      sumr = 0.;
      sumi = 0.;
      i3max = i5+ip3-ip2;
      for (i3=i5; i3 <= i3max; i3 += ip2)
        {
        sumr = sumr+data[i3];
        sumi = sumi+data[i3+1];
        }
      work[1] = sumr;
      work[2] = sumi;
      wr = (*rootr)+1.;
      wi = *rooti;
      iwmin = 1+ip0;
      iwmax = ip0*(((*ifact)+1)/2)-1;
      for (iwork=iwmin; iwork <= iwmax; iwork += ip0)
        {
        twowr = wr+wr;
        i3 = i3max;
        oldsr = 0.;
        oldsi = 0.;
        sumr = data[i3];
        sumi = data[i3+1];
        i3 = i3-ip2;
        doit = 1;
        while (doit == 1 || i3 > i5)
          {
          tempr = sumr;
          tempi = sumi;
          sumr = twowr*sumr-oldsr+data[i3];
          sumi = twowr*sumi-oldsi+data[i3+1];
          oldsr = tempr;
          oldsi = tempi;
          i3 = i3-ip2;
          doit = 0;
          }
        /*  In a Fourier transform the w corresponding to the point at k      */
        /*  is the conjugate of that at ifact-k (that is, exp(twopi*i*        */
        /*  k/ifact) = conj(exp(twopi*i*(ifact-k)/ifact))).  Since the        */
        /*  main loop of Goertzels algorithm is indifferent to the imaginary  */
        /*  part of w, it need be supplied only at the end.                   */
        tempr = -wi*sumi;
        tempi = wi*sumr;
        sumr = wr*sumr-oldsr+data[i3];
        sumi = wr*sumi-oldsi+data[i3+1];
        work[iwork] = sumr+tempr;
        work[iwork+1] = sumi+tempi;
        iwcnj = ip0*((*ifact)+1)-iwork;
        work[iwcnj] = sumr-tempr;
        work[iwcnj+1] = sumi-tempi;
        /* Singleton's recursion, for accuracy and speed (5). */
        tempr = wr;
        wr = wr*(*rootr)-wi*(*rooti)+wr;
        wi = tempr*(*rooti)+wi*(*rootr)+wi;
        }
      iwork = 1;
      for (i3=i5; i3 <= i3max; i3 += ip2)
        {
        data[i3] = work[iwork];
        data[i3+1] = work[iwork+1];
        iwork = iwork+ip0;
        }
      }
    }
  }



/***************************************************************************
c     fixrlx
      subroutine fixrl (data,n,nrem,isign,iform)
c     For iform = 0, convert the transform of a doubled-up real array,
c     considered complex, into its true transform.  Supply only the
c     first half of the complex transform, as the second half has
c     conjugate symmetry.  For iform = -1, convert the first half
c     of the true transform into the transform of a doubled-up real
c     array.  N must be even.
c     Using complex notation and subscripts starting at zero, the
c     transformation is--
c     dimension data(n,nrem)
c     zstp = exp(isign*2*pi*i/n)
c     do 10 i2=0,nrem-1
c     data(0,i2) = conj(data(0,i2))*(1+i)
c     do 10 i1=1,n/4
c     z = (1+(2*iform+1)*i*zstp**i1)/2
c     i1cnj = n/2-i1
c     dif = data(i1,i2)-conj(data(i1cnj,i2))
c     temp = z*dif
c     data(i1,i2) = (data(i1,i2)-temp)*(1-iform)
c 10  data(i1cnj,i2) = (data(i1cnj,i2)+conj(temp))*(1-iform)
c     if i1=i1cnj, the calculation for that value collapses into
c     a simple conjugation of data(i1,i2).
      dimension data(1)
***************************************************************************/

fixrl(data, n, nrem, isign, iform)
  double *data;
  int *n, *nrem, *isign, *iform;
  {
  double twopi, tempr, tempi, sinth, zstpr, zstpi, zr, zi, difr, difi, theta;
  int ip0, ip1, ip2;
  int i1, i2, i1min, i1max, i2min, i2cnj, j1, temp, first;
  temp = (*isign)*(2*(*iform)+1);
  twopi = 6.283185307*((double) (*isign));
  ip0 = 2;
  ip1 = ip0*((*n)/2);
  ip2 = ip1*(*nrem);
  if ((*iform) < 0)
    {
    /* Pack the real input values (two per column). */
    j1 = ip1+1;
    data[2] = data[j1];
    if ((*nrem) > 1)
      {
      j1 = j1+ip0;
      i2min = ip1+1;
      for (i2=i2min; i2 <= ip2; i2 += ip1)
        {
        data[i2] = data[j1];
        j1 = j1+ip0;
        if ((*n) > 2)
          {
          i1min = i2+ip0;
          i1max = i2+ip1-ip0;
          for (i1=i1min; i1 <= i1max; i1 += ip0)
            {
            data[i1] = data[j1];
            data[i1+1] = data[j1+1];
            j1 = j1+ip0;
            }
          }
        data[i2+1] = data[j1];
        j1 = j1+ip0;
        }
      }
    }
  for (i2=1; i2 <= ip2; i2 += ip1)
    {
    tempr = data[i2];
    data[i2] = data[i2]+data[i2+1];
    data[i2+1] = tempr-data[i2+1];
    }
  if ((*n) > 2)
    {
    theta = twopi/((double) (*n));
    sinth = sin(theta/2.);
    zstpr = -2.*sinth*sinth;
    zstpi = sin(theta);
    zr = (1.-zstpi)/2.;
    zi = (1.+zstpr)/2.;
    if ((*iform) < 0)
      {
      zr = 1.-zr;
      zi = -zi;
      }
    i1min = ip0+1;
    i1max = ip0*((*n)/4)+1;
    for (i1=i1min; i1 <= i1max; i1 += ip0)
      {
      for (i2=i1; i2 <= ip2; i2 += ip1)
        {
        i2cnj = ip0*((*n)/2+1)-2*i1+i2;
        if (i2 < i2cnj)
          {
          difr = data[i2]-data[i2cnj];
          difi = data[i2+1]+data[i2cnj+1];
          tempr = difr*zr-difi*zi;
          tempi = difr*zi+difi*zr;
          data[i2] = data[i2]-tempr;
          data[i2+1] = data[i2+1]-tempi;
          data[i2cnj] = data[i2cnj]+tempr;
          data[i2cnj+1] = data[i2cnj+1]-tempi;
          if ((*iform) >= 0) continue;
          data[i2cnj] = data[i2cnj]+data[i2cnj];
          data[i2cnj+1] = data[i2cnj+1]+data[i2cnj+1];
          }
        else
          {
          if (temp < 0) data[i2+1] = -data[i2+1];
          if ((*iform) >= 0) continue;
          }
        data[i2] = data[i2]+data[i2];
        data[i2+1] = data[i2+1]+data[i2+1];
        }
      /* Recursion saves time, at a slight loss in accuracy.  If available, */
      /* Use double precision to compute zr and zi. */
      tempr = zr-.5;
      zr = zstpr*tempr-zstpi*zi+zr;
      zi = zstpr*zi+zstpi*tempr+zi;
      }
    }
  if ((*iform) < 0) return;

  /* Unpack the real transform values (two per column). */
  i2 = ip2+1;
  i1 = i2;
  j1 = ip0*((*n)/2+1)*(*nrem)+1;
  first = 1;
  while (i2 > 1 || first == 1)
    {
    if (first == 0)
      {
      while (i2 < i1)
        {
        data[j1] = data[i1];
        data[j1+1] = data[i1+1];
        i1 = i1-ip0;
        j1 = j1-ip0;
        }
      data[j1] = data[i1];
      data[j1+1] = 0.;
      }
    i2 = i2-ip1;
    j1 = j1-ip0;
    data[j1] = data[i2+1];
    data[j1+1] = 0.;
    i1 = i1-ip0;
    j1 = j1-ip0;
    first = 0;
    }
  data[2] = 0.;
  }



errorstring(string, FORTRANarray, nelements)
  char *string;
  int *FORTRANarray, nelements;
  {
  char temp[30];
  int i, j;
  *string = '\0';
  for (i=1; i <= nelements; i++)
    {
    sprintf(temp, " %d", FORTRANarray[i]);
    for (j=0; temp[j] != '\0'; j++) *string++ = temp[j];
    *string = '\0';
    }
  }







