#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "sherlock.h"
#include "error.h"

long x_offset,y_offset;
extern long totalvals;
extern double maxcorr,*corrmatrix;
extern FILE *corr_resfp;

/*Caculate a frequency domain correlation.  The steps are as follows:

  1) find power of 2 size greater than largest dimension in pattern and image
  2) zero pad pattern and image to that power of 2 size
  3) calculate Fourier Transform of image and pattern
  4) multiply transforms
  5) take inverse of multiplication
  6) search for maximum in inversed transform

  Inputs:  none
  Outputs: none
  Locals:  cols       - number of columns (pixels) for a power of 2 size
	   numpts     - number of points in each dimension to transform
	   nwork      - size of scratch work area (for FFT routine)
	   parray     - pattern data (in space and frequency)
	   rows       - number of rows (scanlines) for a power of 2 size
	   totcols    - number of total columns (pixels) to hold complex data
	   work       - scratch area for FFT routine
  Globals: corrmatrix - matrix to store correlation values
	   errno      - number of last error encountered
	   header     - image/pattern header information arrays
	   totalvals  - total number of correlation values
	   x_offset   - X coordinate offset to get to center of pattern
	   y_offset   - Y coordinate offset to get to center of pattern
	   BATCHMODE  - flag bit indicating Sherlock is running in batch mode
	   FATAL      - error is fatal, program will terminate
	   IMAGE      - reference the image element of an array
	   NULL       - 0
	   PATTERN    - reference the pattern element of an array
*/
freq()
{
	register int numpts[2],nwork;
	register long rows,cols,totcols;
	register double *parray,*work;

	/*Set gauge to 0% complete*/
	if(isbitclear(BATCHMODE)) update_gauge(0.0);

	/*Find power of 2 size for FFT*/
	cols = nint(pow(2.0,ceil(log2((double) header[IMAGE].pixels))));
	rows = nint(pow(2.0,ceil(log2((double) header[IMAGE].scanlines))));
	totcols = cols * 2;
	totalvals = rows * cols * 2;

	/*Find the offsets needed to get to the center of the pattern.  This
	  algorithm finds a match in the image for the upper left corner of
	  the pattern, the offsets are needed to find the coordinate of the
	  center of pattern within the image*/
	x_offset = header[PATTERN].pixels / 2;
	y_offset = header[PATTERN].scanlines / 2;

	/*Multiply power of 2 size by 2 to be able to hold complex data*/
	if(cols > rows) {
	  if((work = (double *) calloc(cols * 2,sizeof(double))) == NULL)
	    error(FATAL,errno);
	  
	  nwork = cols;
	} else {
	         if((work = (double *) calloc(rows * 2,sizeof(double))) == NULL)
	           error(FATAL,errno);

		 nwork = rows;
	       }

	numpts[0] = (int) cols;
	numpts[1] = (int) rows;

	if((corrmatrix = (double *) calloc(rows * totcols,sizeof(double))) ==
	   NULL)
	  error(FATAL,errno);
	if((parray = (double *) calloc(rows * totcols,sizeof(double))) == NULL)
	  error(FATAL,errno);

	if(isbitclear(BATCHMODE)) update_gauge(.125);

	/*Zero pad image data*/
	zero_pad(IMAGE,corrmatrix,cols);
	if(isbitclear(BATCHMODE)) update_gauge(.25);

	/*Zero pad pattern data*/
	zero_pad(PATTERN,parray,cols);
	if(isbitclear(BATCHMODE)) update_gauge(.375);

	/*Transform image from space to frequency domain*/
	fourt(corrmatrix,numpts,2,-1,1,work,nwork);
	if(isbitclear(BATCHMODE)) update_gauge(.5);

	/*Transform pattern from space to frequency domain*/
	fourt(parray,numpts,2,-1,1,work,nwork);
	if(isbitclear(BATCHMODE)) update_gauge(.625);

	/*Multiply transforms*/
	complex_mult(rows,cols,corrmatrix,parray,corrmatrix);
	if(isbitclear(BATCHMODE)) update_gauge(.75);

	/*Take inverse of multiplication result*/
	fourt(corrmatrix,numpts,2,1,1,work,nwork);
	if(isbitclear(BATCHMODE)) update_gauge(.875);

	/*Find maximum of inversed transform*/
	findmax(rows,cols,corrmatrix);
	if(isbitclear(BATCHMODE)) update_gauge(1.0);

	/*Free unneeded space*/
	free((char *) work);
	free((char *) parray);
}

/*Zero pad data.  0's are added after (to the right) and below the data so
  that in the padded matrix, the actual data can be find in the upper left
  corner.  We assume the calloc() call will make every element we don't
  assign a value equal to 0 (since that's it's definition).  We want any
  element not assigned to be 0 (imaginary part is all 0 and the padding is 0).
  
  No artificial high frequencies are introduced that interfere with
  the alogrithm (i.e. no need for a Hanning window).  In addition to creating
  a padded array from the original data, a complex padded array is created
  (the order is real, imaginary, real, imaginary, etc.).

  In order for the frequency domain correlation to work, we need a black (0)
  background with a white (255) object.  Therefore, the first pixel (0,0)
  is checked to see if it's equal to 0.  If it is, we assume the background
  is black and the object is nonblack.  If it isn't, we invert the image (or
  pattern).  This assumes that the pattern won't be covering position (0,0).
  This can be overridden with switches.

  Inputs:  array       - data to be padded
	   cols        - columns (pixels) to pad to
	   rows        - rows (scanlines) to pad to
	   whichimage  - which data is to be padded (image or pattern)
  Outputs: none
  Locals:  aindex      - index into padded data array
	   invert      - flag indicating whether to invert image or not
	   numpadded   - number of pixels to add per scanline
	   xloop       - loop through pixels
	   yloop       - loop through scanlines
  Globals: header      - image/pattern header information arrays
           invert_bias - flag indicating to bias invert operation
	   AUTO_INVERT - automatically determine inversion
	   NO_INVERT   - don't invert
	   YES_INVERT  - invert
*/
zero_pad(whichimage,array,cols)
register int whichimage;
register long cols;
register double *array;
{
	register int invert;
	register long xloop,yloop,aindex,numpadded;

	numpadded = (cols - header[whichimage].pixels) * 2;

	switch(invert_bias[whichimage]) {
	  case AUTO_INVERT:
				if(data[whichimage][cvt(0,0,whichimage)])
	  			  invert = 1;
				else
	  			  invert = 0;

				break;
	  case NO_INVERT:
				invert = 0;

				break;
	  case YES_INVERT:
				invert = 1;

				break;
	}

	if(invert)
	  /*Inversion needed*/
	  for(yloop = aindex = 0;yloop < header[whichimage].scanlines;++yloop) {
	    for(xloop = 0;xloop < header[whichimage].pixels;aindex += 2,++xloop)
	      array[aindex] = (double) (255 - data[whichimage][cvt(xloop,yloop,
							          whichimage)]);

	    aindex += numpadded;
	  }
	else
	  /*No inversion needed*/
	  for(yloop = aindex = 0;yloop < header[whichimage].scanlines;++yloop) {
	    for(xloop = 0;xloop < header[whichimage].pixels;aindex += 2,++xloop)
	      array[aindex] = (double) data[whichimage][cvt(xloop,yloop,
							    whichimage)];

	    aindex += numpadded;
	  }
}

/*Multiply two complex matrices.  Because this is a correlation (as opposed
  to a convolution), the second number is the complex conjugate.

  (a + bi)(c + di) = (ac - bd) + (ad + bc)i
  
  Inputs:  arry1     - complex matrix
	   arry2     - complex matrix
	   cols      - number of columns (pixels) in each matrix
	   result    - resulting matrix from the multiplication
	   rows      - number of rows (scanlines) in each matrix
  Outputs: none
  Locals:  aindex    - loop through complex numbers
	   imag_part - imaginary part of complex multiplication
	   img       - index of imaginary part of complex number
	   ivalue    - value of 2nd complex number's imaginary part
	   real_part - real part of complex multiplication
  Globals: none
*/
complex_mult(rows,cols,arry1,arry2,result)
register long rows,cols;
register double *arry1,*arry2,*result;
{
	register long aindex,img;
	register double ivalue,real_part,imag_part;

	for(aindex = 0;aindex < (rows * cols * 2);aindex += 2) {
	  img = aindex + 1;
	  ivalue = -arry2[img];		/*Complex conjugate*/

	  real_part = (arry1[aindex] * arry2[aindex]) - (arry1[img] * ivalue);

	  imag_part = (arry1[aindex] * ivalue) + (arry1[img] * arry2[aindex]);

	  /*Put real and imaginary parts of answer into result matrix*/
	  result[aindex] = real_part;
	  result[img] = imag_part;

	}
}

/*Find a maximum in a 2D matrix.

  Inputs:  array      - matrix to search
	   cols       - number of columns (pixels) in matrix
	   rows       - number of rows (scanlines) in matrix
  Outputs: none
  Locals:  aindex     - loop through matrix
  Globals: corr_resfp - file pointer to correlation graph data
	   info       - structure containing results of the algorithm
	   maxcorr    - maximum correlation value
	   x_offset   - X coordinate offset to get to center of pattern
	   y_offset   - Y coordinate offset to get to center of pattern
*/
findmax(rows,cols,array)
register long rows,cols;
register double *array;
{
	register long aindex;

	/*Set to 1st number in matrix*/
	maxcorr = array[0];
	info.x = 0;
	info.y = 0;

	/*Start searching from the 2nd number on*/
	for(aindex = 2;aindex < (rows * cols * 2);aindex += 2) {
	  /*If we find a bigger number than what we have, store it*/
	  if(array[aindex] > maxcorr) {
	    maxcorr = array[aindex];

	    /*The position of the new maximum has the offset added to get
	      to the center of the pattern.  This maximum from this algorithm
	      will be at the upper left corner of the pattern*/
	    info.x = (aindex / 2) % cols + x_offset;
	    info.y = (aindex / 2) / cols + y_offset;
	  }
	}
}

