#include <stdlib.h>
#include <ctype.h>
#include <xview/cms.h>
#include <xview/font.h>
#include "ui.h"
#include "ui_globals.h"
#include "sherlock.h"
#include "sherlock.xbm"
#include "error.h"

char *loadsave_label[] = {"Load Image","Load Pattern","Save Pattern"};
Cms cms;

/*Initialize variables, UI, etc.

  Inputs:  argc                  - number of command line arguments
	   argv                  - command line arguments
  Outputs: batchfp               - file pointer to file containing batch
				   commands
  Locals:  batchfp               - file pointer to file containing batch
				   commands
	   err                   - error from readheader (if there is one)
	   fd                    - file descriptor of image/pattern data files
  Globals: BATCHMODE             - flag bit indicating Sherlock is running in
				   batch mode
	   DISPLAY_POS           - display platten menu item position
	   FATAL                 - error is fatal, program will terminate
	   FILE_MENU_HANDLE      - handle to UI "File" menu button
	   IMAGE                 - reference the image element of an array
	   NULL		         - 0
	   PATTERN               - reference the pattern element of an array
	   PATTERN_MENU_HANDLE   - handle to UI "Pattern" menu button
	   RUN_BUTTON_HANDLE     - handle to UI "Run" menu button
	   SAVEPATTERN_POS       - save pattern menu item position
	   SIGINT                - user interrupt signal
	   XV_INIT_ARGC_PTR_ARGV - initialize xview toolkit
*/
FILE *init(argc,argv)
int argc;
register char *argv[];
{
	register int fd[2],err;
	register FILE *batchfp;
	int cleanup();
	FILE *getargs();

	/*Set file creation mask*/
	umask(022);

	/*Capture software interrupt to clean up properly*/
	signal(SIGINT,cleanup);

	/*Initialize variables*/
	init_vars();

	/*Parse arguments*/
	batchfp = getargs(argc,argv,fd);

	/*Initialize xview if not in batch mode*/
	if(isbitclear(BATCHMODE))
	  xv_init(XV_INIT_ARGC_PTR_ARGV,&argc,argv,NULL);

	/*Read image/pattern headers*/
	if((err = readheader(fd[IMAGE],IMAGE))) error(FATAL,err);
	if(fd[PATTERN] != -1)
	  if((err = readheader(fd[PATTERN],PATTERN))) error(FATAL,err);

	/*Initialize UI*/
	if(isbitclear(BATCHMODE)) {
	  init_frames(fd[PATTERN]);

	  init_loadsave();

	  init_panel();

	  init_panel_objects(fd[PATTERN]);

	  init_cmap();

	  init_canvas();
	}

	/*Read image data*/
	readimage(fd[IMAGE],IMAGE,NULL,(HEADER *) NULL);

	/*Read pattern data (if there is any)*/
	if(fd[PATTERN] != -1) {
	  readimage(fd[PATTERN],PATTERN,NULL,(HEADER *) NULL);

	  /*Activate "pattern" buttons if pattern data read in*/
	  if(isbitclear(BATCHMODE)) {
	    activate(FILE_MENU_HANDLE,SAVEPATTERN_POS);
	    activate(PATTERN_MENU_HANDLE,DISPLAY_POS);
	    activate(RUN_BUTTON_HANDLE,0);
	  }
	}

	return(batchfp);
}

/*Initialize global variables.

  Inputs:  none
  Outputs: none
  Locals:  loop         - loop through load/save filename browser windows
  Globals: chdir        - change in directory for browser windows
	   chfile       - change in filename for browser windows
	   crop_box     - crop area coordinates
	   data         - un(color)mapped image data
	   dir          - directory set in the browser windows
	   file         - filename set in the browser windows
	   frame        - UI window handles
	   image        - array containing image and pattern data
	   info         - structure containing results of the algorithm
	   pattern_info - pattern location information
*/
init_vars()
{
	register int loop;
	extern char *dir[],*file[],*chdir[],*chfile[];
	extern XRectangle crop_box;

	data[IMAGE] = data[PATTERN] = NULL;

	frame[IMAGE] = frame[PATTERN] = NULL;
	image[IMAGE] = image[PATTERN] = NULL;

	filename[IMAGE] = filename[PATTERN] = NULL;

	pattern_info.xoffset = pattern_info.yoffset = 0;

	crop_box.x = crop_box.y = -1;

	info.alg = info.seed = -1;
	info.p1 = 0;

	for(loop = 0;loop < NUM_SCREENS;++loop)
	  dir[loop] = file[loop] = chdir[loop] = chfile[loop] = NULL;
}

/*Initialize UI window frames.

  Inputs:  havepattern          - flag indicating we have pattern data to
				  display
  Outputs: none
  Locals:  icon                 - window icon image
	   label                - base frame label
  Globals: errno                - number of last error encountered
	   frame                - UI window handles
	   header               - image/pattern header information arrays
	   program              - name of currently executing program
	   FALSE                - turn attribute off
	   FATAL                - error is fatal, program will terminate
	   FRAME                - frame window object
	   FRAME_CMD            - pinup subwindow object
	   FRAME_ICON           - icon image attribute
	   FRAME_INHERIT_COLORS - child should inherit parent's colormap attr
	   FRAME_LABEL          - window label attribute
	   FRAME_SHOW_FOOTER    - show footer strings attribute
	   GAUGE                - reference the xview gauge object of an array
	   IMAGE                - reference the image element of an array
	   NULL			- 0
	   PANEL_HEIGHT		- height of control panel
	   PATTERN              - reference the pattern element of an array
	   RESULTS              - reference the results window object of an
				  array
	   TRUE			- turn an attribute on
	   VERSION		- version number
	   XV_HEIGHT            - xview object height attribute
	   XV_SHOW		- map a window to the display (i.e. display
				  window)
	   XV_WIDTH		- xview object width attribute
*/
init_frames(havepattern)
{
	register char *label;
	Icon icon;

	/*Label is the size of the version number string + blank
	  space between that and program name + program name + one for
	  the ending NULL*/
	if((label = malloc(strlen(program) + strlen(VERSION) + 2)) == NULL)
	  error(FATAL,errno);
	else {
		 sprintf(label,"%s %s",program,VERSION);

		 /*Yes, I like the first letter capitalized, just call me
		   fussy*/
		 if(islower(label[0])) label[0] = toupper(label[0]);
	       }

	/*Create icon image from bitmap for base frame*/
	create_icon(&icon,label);

	/*Create the base frame*/
	frame[IMAGE] = (Frame) xv_create(NULL,FRAME,
				         FRAME_LABEL,label,
				         FRAME_SHOW_FOOTER,TRUE,
				         FRAME_ICON,icon,
					 FRAME_INHERIT_COLORS,TRUE,
				         NULL);

	/*Some X servers couldn't handle setting the size of the frame in
	  it's creation, so it's done after*/
	xv_set(frame[IMAGE],
	       XV_WIDTH,header[IMAGE].pixels,
	       XV_HEIGHT,PANEL_HEIGHT + header[IMAGE].scanlines,
	       XV_SHOW,TRUE,
	       NULL);

	/*Create icon image from bitmap for pattern display window*/
	create_icon(&icon,"pattern");

	/*Create pattern display frame*/
	frame[PATTERN] = (Frame) xv_create(frame[IMAGE],FRAME,
					   FRAME_LABEL,"Pattern",
					   FRAME_SHOW_FOOTER,TRUE,
					   FRAME_ICON,icon,
					   XV_SHOW,FALSE,
					   NULL);

	/*Indicate to the user what's happening*/
	set_footer("Initializing...",IMAGE);

	free(label);

	/*Make sure server has at least an 8 bit plane display*/
	check_server();

	/*Create frame for pattern to be displayed in*/
	if(havepattern != -1)
	  xv_set(frame[PATTERN],XV_WIDTH,header[PATTERN].pixels,
		 XV_HEIGHT,header[PATTERN].scanlines,XV_SHOW,TRUE,NULL);

	/*We have to show this subframe and them immediately unmap it so that
	  the gauge panel item will show up when we DO want it*/
	frame[GAUGE] = (Frame) xv_create(frame[IMAGE],FRAME_CMD,
					 FRAME_LABEL,"Percent Complete",
					 XV_WIDTH,300,
					 XV_HEIGHT,50,
					 NULL);
	xv_set(frame[GAUGE],XV_SHOW,TRUE,XV_SHOW,FALSE,NULL);

	/*Create results window*/
	frame[RESULTS] = (Frame) xv_create(frame[IMAGE],FRAME_CMD,
					   FRAME_LABEL,"Test Results",
					   XV_WIDTH,290,
					   XV_HEIGHT,170,
					   NULL);
}

/*Initialize load/save dialog boxes.

  Inputs:  none
  Outputs: none
  Locals:  cmdpanel                   - handle to subwindow's panel object
	   item                       - handle to panel item
	   loop                       - loop through dialog boxes
  Globals: frame                      - UI window handles
	   loadsave                   - load/save dialog box UI handles
	   loadsave_label             - array of lables for dialog boxes
	   APPLY_HANDLE               - handle to UI "Apply" button
	   CANCEL_HANDLE              - handle to UI "Cancel" button
	   DIR_HANDLE                 - handle to UI directory panel item
	   FALSE                      - turn attribute off
	   FILE_HANDLE                - handle to UI filename panel item
	   FRAME_CMD                  - frame's command panel UI handle
	   FRAME_CMD_PANEL            - frame's command panel UI handle
	   FRAME_LABEL                - window label attribute
	   NULL                       - 0
	   NUM_SCREENS                - number of load/save dialog boxes
	   PANEL_BUTTON               - panel button object
	   PANEL_CLIENT_DATA          - data attached to an xview object
	   PANEL_LABEL_STRING         - panel window label attribute
	   PANEL_NOTIFY_PROC          - notification procedure attribute
	   PANEL_TEXT                 - panel text attribute
	   PANEL_VALUE_DISPLAY_LENGTH - length of panel field attribute
	   XV_HEIGHT                  - xview object height attribute
	   XV_KEY_DATA		      - data attached to an xview object
	   XV_SHOW                    - map a window to the display
					(i.e. display window)
	   XV_WIDTH                   - xview object width attribute
	   XV_Y                       - Y coordinate attribute
*/
init_loadsave()
{
	register int loop;
	register Panel cmdpanel;
	register Panel_item item;
	Panel_setting get_browser_text();
	void loadsave_exit();

	/*Create load/save dialog boxes*/
	for(loop = 0;loop < NUM_SCREENS;++loop) {
	  /*Create window frame*/
	  loadsave[loop] = (Frame) xv_create(frame[IMAGE],FRAME_CMD,
					     FRAME_LABEL,loadsave_label[loop],
				       	     XV_WIDTH,350,
				       	     XV_HEIGHT,180,
				       	     XV_SHOW,FALSE,
				       	     NULL);

	  /*Create directory input object*/
	  cmdpanel = (Panel) xv_get(loadsave[loop],FRAME_CMD_PANEL);
	  item = (Panel_item) xv_create(cmdpanel,PANEL_TEXT,
		  		        PANEL_LABEL_STRING,"Directory:",
		  		        PANEL_NOTIFY_PROC,get_browser_text,
		  		        PANEL_VALUE_DISPLAY_LENGTH,30,
		  		        PANEL_CLIENT_DATA,0,
		  		        XV_Y,20,
		  		        NULL);
	  xv_set(loadsave[loop],XV_KEY_DATA,DIR_HANDLE,item,NULL);
  
	  /*Create filename input object*/
	  item = (Panel_item) xv_create(cmdpanel,PANEL_TEXT,
		  		        PANEL_LABEL_STRING,"File:",
		  		        PANEL_NOTIFY_PROC,get_browser_text,
		  		        PANEL_VALUE_DISPLAY_LENGTH,30,
		  		        PANEL_CLIENT_DATA,1,
		  		        XV_X,42,
		  		        XV_Y,60,
		  		        NULL);
	  xv_set(loadsave[loop],XV_KEY_DATA,FILE_HANDLE,item,NULL);

	  /*Create apply button*/
	  item = (Panel_item) xv_create(cmdpanel,PANEL_BUTTON,
		  		        PANEL_LABEL_STRING,"Apply",
		  		        PANEL_NOTIFY_PROC,loadsave_exit,
		  		        XV_X,95,
		  		        XV_Y,130,
		  		        NULL);
	  xv_set(loadsave[loop],XV_KEY_DATA,APPLY_HANDLE,item,NULL);

	  /*Create cancel button*/
	  item = (Panel_item) xv_create(cmdpanel,PANEL_BUTTON,
                  		        PANEL_LABEL_STRING,"Cancel",
                  		        PANEL_NOTIFY_PROC,loadsave_exit,
                  		        XV_X,195,
                  		        XV_Y,130,
                  		        NULL);
	  xv_set(loadsave[loop],XV_KEY_DATA,CANCEL_HANDLE,item,NULL);
	}
}


/*Create command panel.

  Inputs:  none
  Outputs: none
  Locals:  none
  Globals: frame        - UI window handles
	   panel	- handle to command panel
	   IMAGE        - reference the image element of an array
	   NULL         - 0
	   PANEL        - xview panel object
	   PANEL_HEIGHT - height of control panel
	   XV_HEIGHT    - xview object height attribute
*/
init_panel()
{
	panel = (Panel) xv_create(frame[IMAGE],PANEL,
				  XV_HEIGHT,PANEL_HEIGHT,
				  NULL);

}

/*Initialize all panel objects.

  Inputs:  havepattern         - flag indicating we have pattern data to display
  Outputs: none
  Locals:  item                - handle to panel items
	   menu                - handle to panel menus
	   subpanel            - handle to base frame's panel
  Globals: panel               - handle to command panel
	   FILE_MENU_HANDLE    - handle to UI "File" menu button
	   FRAME_CMD_PANEL     - frame's command panel UI handle
	   GAUGE               - reference the xview gauge object of an array
	   GAUGE_HANDLE        - handle to UI gauge panel item
	   MENU                - menu object
	   MENU_ACTION_ITEM    - associate a menu item with a callback function
	   MENU_INACTIVE       - grayed out attribute
	   MENU_ITEM           - menu attributes
	   MENU_NOTIFY_PROC    - menu notification function attribute
	   MENU_STRING         - menu choice string attribute
	   MENU_STRINGS        - multiple menu choice strings attribute
	   NULL                - 0
	   PANEL_GAUGE         - panel gauge objext
	   PANEL_GAUGE_WIDTH   - gauge width attribute
	   PANEL_HORIZONTAL    - lay object horizontally
	   PANEL_LAYOUT        - object layout attribute
	   PANEL_MAX_VALUE     - maximum value for panel object
	   PANEL_MIN_VALUE     - minimum value for panel object
	   PANEL_TICKS         - number of ticks for gauge attribute
	   PANEL_VALUE         - object's value attribute
	   PATTERN_MENU_HANDLE - handle to UI "Pattern" menu button
	   RUN_BUTTON_HANDLE   - handle to UI "Run" menu button
	   SAVEPATTERN_POS     - save pattern menu item position
	   XV_KEY_DATA         - data attached to an xview object
	   XV_X                - X coordinate attribute
	   XV_Y                - Y coordinate attribute
*/
init_panel_objects(havepattern)
register int havepattern;
{
	register Menu menu;
	register Panel subpanel;
	register Panel_item item;
	void show_filename(),quit(),select_pattern(),display_pattern();
	void get_alg();

	/*Create "File" button menu*/
	menu = (Menu) xv_create(NULL,MENU,
				MENU_STRINGS,"Load Image...",
				    "Load Pattern...","","Save Pattern...",
				    "",NULL,
				MENU_NOTIFY_PROC,show_filename,
				MENU_ACTION_ITEM,"Exit",quit,
				NULL);
	createbutton("File");
	xv_set(panel,XV_KEY_DATA,FILE_MENU_HANDLE,menu,NULL);
	/*Deactivate menu whitespace and Save Pattern, positions 3 and 5
	  correspond to the whitespace*/
	deactivate(FILE_MENU_HANDLE,3);
	if(havepattern == -1) deactivate(FILE_MENU_HANDLE,SAVEPATTERN_POS);
	deactivate(FILE_MENU_HANDLE,5);

	/*Create "Pattern" button menu*/
	menu = (Menu) xv_create(NULL,MENU,
				MENU_ITEM,
				    MENU_STRING,"Select",
				    MENU_NOTIFY_PROC,select_pattern,
				    MENU_INACTIVE,TRUE,
				    NULL,
				MENU_ITEM,
				    MENU_STRING,"Display",
				    MENU_NOTIFY_PROC,display_pattern,
				    MENU_INACTIVE,TRUE,
				    NULL,
				NULL);

	createbutton("Pattern");
	xv_set(panel,XV_KEY_DATA,PATTERN_MENU_HANDLE,menu,NULL);

	/*Create "Run" button menu*/
	menu = (Menu) xv_create(NULL,MENU,
				MENU_STRINGS,"Correlation (space domain)",
		                    "Correlation (frequency domain)",
		                    "Linear Pixel Shuffling",
				    "Random Ordering",NULL,
				MENU_NOTIFY_PROC,get_alg,
				NULL);

	item = (Panel_item) createbutton("Run");
	xv_set(panel,XV_KEY_DATA,RUN_MENU_HANDLE,menu,
	       XV_KEY_DATA,RUN_BUTTON_HANDLE,item,NULL);
	deactivate(RUN_BUTTON_HANDLE,0);

	/*Create gauge*/
	subpanel = (Panel) xv_get(frame[GAUGE],FRAME_CMD_PANEL);
	xv_set(subpanel,WIN_BORDER,TRUE,NULL);
	item = (Panel_item) xv_create(subpanel,PANEL_GAUGE,
				      PANEL_VALUE,0,
				      PANEL_MIN_VALUE,0,
				      PANEL_MAX_VALUE,100,
				      PANEL_LAYOUT,PANEL_HORIZONTAL,
				      PANEL_TICKS,21,
				      PANEL_GAUGE_WIDTH,200,
				      XV_X,40,
				      XV_Y,15,
				      NULL);
	xv_set(frame[GAUGE],XV_KEY_DATA,GAUGE_HANDLE,item,NULL);
}


/*Initialize the grayscale colormap.

  Inputs:  none
  Outputs: none
  Locals:  colors          - RGB values for 256 grayscale values
	   inc             - amount to incrememt for 256 values;
	   loop            - loop through colormap entries
	   ncells          - number of cells in the display
	   screen          - X display screen
	   screen_no       - screen number
	   values          - graphic context values
  Globals: cmlut           - colormap lookup table
	   cms             - colormap
	   dpy             - X display
	   frame           - UI window handles
	   norm_gc	   - normal graphics context
	   xor_gc	   - exclusive or graphics context to draw cropping box with
	   CMS             - colormap object
	   CMS_COLORS      - assign RGB values to make a colormap
	   CMS_INDEX_TABLE - lookup table attribute
	   CMS_SIZE        - number of entries in colormap attribute
	   CMS_TYPE        - type of colormap attribute
	   GCForeground    - window foreground color attribute
	   GCFunction      - drawing function attribute
	   GXxor           - drawing function (exclusive or)
	   IMAGE           - reference the image element of an array
	   NULL            - 0
	   SCREEN_NUMBER   - screen number attribute
	   XV_SCREEN       - xview screen attribute
	   XV_STATIC_CMS   - create a static colormap
*/
init_cmap()
{
	register int ncells,loop,inc,screen_no;
	XGCValues values;
	register Xv_Screen screen;
	Xv_Singlecolor colors[256];
	extern GC norm_gc,xor_gc;

	/*Find out how many cells this X server is capable of*/
	ncells = DisplayCells(dpy,DefaultScreen(dpy));
	inc = 256 / ncells;

	/*Create a grayscale lookup table*/
	for(loop = 0;loop < ncells;loop += inc)
	  colors[loop].red = colors[loop].blue = colors[loop].green = loop;

	cms = xv_create(0,CMS,
			CMS_TYPE,XV_STATIC_CMS,
			CMS_SIZE,ncells,
			NULL);

	/*Set grayscale colormap*/
	xv_set(cms,CMS_COLORS,colors,NULL);

	/*Get lookup table for grayscale colormap*/
	cmlut = (unsigned long *) xv_get(cms,CMS_INDEX_TABLE);

	screen = (Xv_Screen) xv_get(frame[IMAGE],XV_SCREEN);
	screen_no = (int) xv_get(screen,SCREEN_NUMBER);

	/*Get normal graphics context*/
	norm_gc = DefaultGC(dpy,screen_no);

	/*Create graphics context for drawing cropping box*/
	values.function = GXxor;
	values.foreground = cmlut[127];
	xor_gc = XCreateGC(dpy,RootWindow(dpy,screen_no),
			   (GCForeground | GCFunction),&values);
}

/*Initialize canvases.

  Inputs:  none
  Outputs: none
  Locals:  font                           - font for results window
	   gc                             - graphics context
	   gcvalues                       - graphics context values
	   loop                           - loop through display canvases
  Globals: canvas                         - UI canvas handles (that display
					    the images and results)
	   cms                            - colormap
	   frame                          - UI window handles
	   rcanvas                        - canvs to display algorithm results
	   CANVAS                         - canvas object
	   CANVAS_AUTO_CLEAR 		  - automatically clear on redraw
					    attribute
	   CANVAS_REPAINT_PROC 		  - repaint procedure attribute
	   FALSE               		  - turn attribute off
	   FONT                		  - font object
	   FONT_FAMILY         		  - font family attribute
	   FONT_FAMILY_DEFAULT_FIXEDWIDTH - server's default fixed width font
	   FONT_SCALE                     - size of font
	   False               		  - turn attribute off
	   GCBackground                   - background color attribute
	   GCFont                         - font attribute
	   GCForeground                   - foreground color attribute
	   GCGraphicsExposures            - graphics exposures attribute
	   GC_HANDLE                      - handle to graphics context
	   LOC_DRAG                       - capture mouse dragging events
	   LOC_MOVE			  - capture mouse moving events
	   NULL				  - 0
	   RESULTS                        - reference the results window
					    object of an array
	   WIN_CLIENT_DATA		  - data attached to an xview object
	   WIN_CMS                        - colormap attribute
	   WIN_CONSUME_EVENTS             - capture these events
	   WIN_EVENT_PROC                 - callback for events
	   WIN_MOUSE_BUTTONS              - capture mouse events
	   WIN_NO_EVENTS                  - capture no events
	   WIN_REPAINT                    - capture repaint event
	   WIN_SCALE_MEDIUM               - medium size font
	   XID_HANDLE                     - handle to X ID value
	   XV_FONT                        - font object
	   XV_KEY_DATA			  - data attached to an xview object
	   XV_X				  - X coordinate attribute
	   XV_XID                         - X ID value
	   XV_Y				  - Y coordinate attribute
*/
init_canvas()
{
	register int loop;
	register GC gc;
	XGCValues gcvalues;
	register Xv_Font font;
	void repaint(),mouse_event(),display_results();

	/*Loop through display canvases*/
	for(loop = 0;loop < 2;++loop) {
	  /*Create canvas*/
	  canvas[loop] = (Canvas) xv_create(frame[loop],CANVAS,
				            CANVAS_AUTO_CLEAR,FALSE,
				            CANVAS_REPAINT_PROC,repaint,
				            WIN_CMS,cms,
				            NULL);
	  /*Store X ID value*/
	  xv_set(canvas[loop],
		 WIN_CLIENT_DATA,xv_get(canvas_paint_window(canvas[loop]),
					XV_XID),
		 NULL);

	  /*Set up capturing of canvas events*/
	  xv_set(canvas_paint_window(canvas[loop]),
		 WIN_EVENT_PROC,mouse_event,
		 WIN_CONSUME_EVENTS,
		   WIN_NO_EVENTS,WIN_REPAINT,LOC_MOVE,LOC_DRAG,
		   WIN_MOUSE_BUTTONS,NULL,
		 NULL);

	  window_fit(canvas[loop]);
	}

	/*Create results window canvas*/
	rcanvas = (Canvas) xv_create(frame[RESULTS],CANVAS,
				     CANVAS_REPAINT_PROC,display_results,
				     XV_X,0,
				     XV_Y,0,
				     NULL);
	window_fit(frame[RESULTS]);

	/*Get font for results window*/
	if(!(font = (Xv_Font) xv_find(rcanvas,FONT,
		  		     FONT_FAMILY,FONT_FAMILY_DEFAULT_FIXEDWIDTH,
		  		     FONT_SCALE,WIN_SCALE_MEDIUM,
		  		     NULL)))
	  font = (Xv_Font) xv_get(canvas,XV_FONT);

	/*Set up graphics context for results window*/
	gcvalues.font = (Font) xv_get(font,XV_XID);
	gcvalues.foreground = BlackPixel(dpy,DefaultScreen(dpy));
	gcvalues.background = WhitePixel(dpy,DefaultScreen(dpy));
	gcvalues.graphics_exposures = False;
	gc = XCreateGC(dpy,RootWindow(dpy,DefaultScreen(dpy)),
		       GCForeground | GCBackground | GCFont |
		       GCGraphicsExposures,&gcvalues);

	xv_set(frame[RESULTS],
	       XV_KEY_DATA,GC_HANDLE,gc,
	       XV_KEY_DATA,XID_HANDLE,xv_get(canvas_paint_window(rcanvas),
					     XV_XID),
	       NULL);
}


/*Check the server's display to make sure we have at least 8 planes (for
  grayscale).

  Inputs:  none
  Outputs: none
  Locals:  screen        - X display screen
	   screen_no     - screen number
  Globals: dpy           - X display
	   frame         - UI window handles
	   ENODEPTH      - "display has less than 8 planes" error
	   FATAL         - error is fatal, program will terminate
	   IMAGE         - reference the image element of an array
	   SCREEN_NUMBER - screen number attribute
	   XV_DISPLAY    - X display attribute
	   XV_SCREEN     - xview screen attribute
*/
check_server()
{
	register int screen_no;
	register Xv_Screen screen;

	dpy = (Display *) xv_get(frame[IMAGE],XV_DISPLAY);
	screen = (Xv_Screen) xv_get(frame[IMAGE],XV_SCREEN);
	screen_no = (int) xv_get(screen,SCREEN_NUMBER);

	if(DefaultDepth(dpy,screen_no) < 8) error(FATAL,ENODEPTH);
}

/*Create icon image from a bitmap.

  Inputs:  label               - lable to put into icon
  Outputs: icon                - icon image
  Locals:  dims                - font dimensions
	   icon_font           - icon font
	   icon_image          - icon image
	   image_rect          - size and position information for the icon
				 image
	   label_rect          - size and position information for the icon
				 label
  Globals: sherlock_height     - height of bitmap image
	   sherlock_width      - width of bitmap image
	   DEF_ICON_FONT       - default icon font to use
	   FONT                - font object
	   FONT_NAME           - name of font attribute
	   FONT_STRING_DIMS    - dimensions of a string in a certain font
	   ICON                - icon object
	   ICON_FONT           - icon font attribute
	   ICON_HEIGHT_ADD     - label height
	   ICON_IMAGE          - icon image attribute
	   ICON_IMAGE_RECT     - icon image size & position attribute
	   ICON_LABEL          - icon label attribute
	   ICON_LABEL_RECT     - icon label size & position attribute
	   ICON_WIDTH_ADD      - amount to add to width to make image look good
	   NULL		       - 0
	   SERVER_IMAGE        - server icon image object
	   SERVER_IMAGE_DEPTH  - number of bits per pixel attribute
	   SERVER_IMAGE_X_BITS - bitmap values attribute
	   XV_HEIGHT           - xview object height attribute
	   XV_WIDTH            - xview object width attribute
*/
create_icon(icon,label)
register char *label;
register Icon *icon;
{
	Font_string_dims dims;
	Rect image_rect,label_rect;
	register Server_image icon_image;
	register Xv_Font icon_font;

	/*Create the icon font*/
	icon_font = (Xv_Font) xv_find(NULL,FONT,
				      FONT_NAME,DEF_ICON_FONT,
				      NULL);

	xv_get(icon_font,FONT_STRING_DIMS,label,&dims);
	/*There's a bug in Xview 2.0/3.0 that the X position of the icon
	  label has no effect, so don't bother with it*/
	rect_construct(&label_rect,50,sherlock_height + ICON_HEIGHT_ADD,
		       sherlock_width,sherlock_height);

	/*Create the icon image*/
	icon_image = (Server_image) xv_create(NULL,SERVER_IMAGE,
					      XV_WIDTH,sherlock_width,
					      XV_HEIGHT,sherlock_height,
					      SERVER_IMAGE_DEPTH,1,
					      SERVER_IMAGE_X_BITS,sherlock_bits,
					      NULL);

	/*Create the icon*/
	rect_construct(&image_rect,5,0,sherlock_width,sherlock_height);
	*icon = (Icon) xv_create(NULL,ICON,
			         XV_WIDTH,sherlock_width + ICON_WIDTH_ADD,
				 XV_HEIGHT,sherlock_height + ICON_HEIGHT_ADD +
				 dims.height,
				 ICON_FONT,icon_font,
				 ICON_LABEL,label,
				 ICON_LABEL_RECT,&label_rect,
				 ICON_IMAGE,icon_image,
				 ICON_IMAGE_RECT,&image_rect,
				 NULL);
}

