#include <values.h>
#include "sherlock.h"

/*Calculate a space domain correlation using Linear Pixel Shuffling.  The
  steps are as follows:

  1) find 2 maximum numbers equal to or greater than the size of the image
  2) start at [0,0]
  3) calculate correlation
  4) see if this has a minimum value
  5) find next position and calculate correlation
  6) see if this has a minimum value
  7) repeat steps 5 and 6 until the entire image has been covered

  Because of the way the correlation is calculated, (it's actually a
  difference rather than a correlation) we look for the smallest
  value, rather than the largest.

  Inputs:  none
  Outputs: none
  Locals:  loop      - loop until all pixels in image covered
	   min       - minimum correlation value
	   pos       - position to store correlation value in array
	   res       - result of correlation
	   total     - total number of pixels in image
	   xcoord    - X coordinate to correlate at
	   ycoord    - Y coordinate to correlate at
  Globals: header    - image/pattern header information arrays
	   info      - structure containing results of the algorithm
	   BATCHMODE - flag bit indicating Sherlock is running in batch mode
	   IMAGE     - reference the image element of an array
	   MAXDOUBLE - maximum value of a double variable
*/
lps()
{
	register long loop,xcoord,ycoord,total,pos;
	register double min,res;

	/*Calculate maximum numbers using modified Fibonacci sequence*/
	if(!info.p1)
	  if(header[IMAGE].pixels > header[IMAGE].scanlines)
	    calc_fibonacci(header[IMAGE].pixels,&info.p1,&info.p2,&info.k1,
			   &info.k2);
	  else
	    calc_fibonacci(header[IMAGE].scanlines,&info.p1,&info.p2,&info.k1,
			   &info.k2);

	/*Set initial conditions*/
	min = MAXDOUBLE;
	xcoord = ycoord = 0;
	total = info.p1 * info.p2;

	/*Loop through pixels*/
	for(loop = pos = 0;loop < total;++loop) {
	  if(isbitclear(BATCHMODE) && !(loop % header[IMAGE].pixels))
	    update_gauge((double) loop / (double) total);

	  /*Calculate correlation and compare for minimum*/
	  if((xcoord < header[IMAGE].pixels) &&
	     (ycoord < header[IMAGE].scanlines))
	    if((res = corr(xcoord,ycoord,pos++)) < min) {
	      min = res;
	      info.x = xcoord;
	      info.y = ycoord;
	    }

	  /*Determine next pixel*/
	  xcoord = (xcoord + info.k1) % info.p1;
	  ycoord = (ycoord + info.k2) % info.p2;
	}
}

