#include <stdio.h>
#include <sys/time.h>
#include "ui.h"
#include "ui_globals.h"
#include "sherlock.h"
#include "error.h"
#include "timing/timing.h"

long sec,totalvals;
double maxcorr,*corrmatrix;
extern FILE *corr_resfp;

/*Run the user selected algorithm.  The execution involves the parent process
  running the alogrithm.  The child timer process is instructed to start
  signaling the parent every "so many clicks" of time.  When the parent
  receives the signal, it sends the position it thinks the pattern is to
  the child and continues processing.  When done, it tells the child to
  wrapup this session.

  Inputs:  alg              - algorithm to run
  Outputs: none
  Locals:  dataset          - current dataset this algorithm will produce
	   loop		    - loop through correlation matrix
	   string           - commands sent to timer process
  Globals: corr_resfp       - file pointer to correlation graph data
  	   corrmatrix       - matrix to hold correlation values
	   errno	    - number of last error encountered
	   frame            - UI window handles
	   header	    - image/pattern header information arrays
	   info             - structure containing results of the algorithm
	   maxcorr          - maximum correlation value
	   sec              - counter of how many "clicks" of time have gone by
	   totalvals	    - total number of correlation values
	   writetimer       - file descriptor to write to child timer process
	   BATCHMODE        - flag bit indicating Sherlock is running in batch
			      mode
	   BEGINTIMING      - macro to record start time
	   ENDTIMING        - macro to calculate time spent
	   FATAL	    - error is fatal, program will terminate
	   FREQ_DOMAIN      - run frequency domain correlation
	   IMAGE            - reference the image element of an array
	   LPS              - run linear pixel shuffling
	   NOTIFY_ASYNC     - notify signal process asynchronously
	   NOTIFY_FUNC_NULL - stop calling a callback function for a signal
	   NULL		    - 0
	   PATTERN          - reference the pattern element of an array
	   RANDOM           - run random ordering
	   SIGUSR1          - user defined signal #1
	   SIG_DFL          - set up default signal action
	   SPACE_DOMAIN     - run space domain correlation
	   TIMINGS          - time spent in seconds
*/
run(alg)
register int alg;
{
	static int dataset = 0;
	register int loop;
	register char string[20];
	void checkacc();

	/*Set up initial conditions*/
	info.alg = alg;
	info.x = info.y = 0;
	sec = 1L;
	maxcorr = 0.0;

	/*Allocate space to hold correlation values*/
	if(alg != FREQ_DOMAIN) {
	  if((corrmatrix = (double *) calloc(header[IMAGE].pixels *
					     header[IMAGE].scanlines,
					     sizeof(double))) == NULL)
	    error(FATAL,errno);

	  totalvals = header[IMAGE].pixels * header[IMAGE].scanlines;
	}

	/*Set up to catch signal sent from child timer process*/
	if(isbitclear(BATCHMODE))
	  notify_set_signal_func(frame[IMAGE],checkacc,SIGUSR1,NOTIFY_ASYNC);
	else signal(SIGUSR1,checkacc);

	/*Tell the child to start sending signals to the parent*/
	sprintf(string,"s %d %ld %ld\n",alg,header[PATTERN].xcenter,
		header[PATTERN].ycenter);
	write(writetimer,string,strlen(string));

	/*Start timer*/
	BEGINTIMING;

	/*Run algorithm*/
	switch(alg) {
	  case SPACE_DOMAIN:
				space();
				break;
	  case FREQ_DOMAIN:
				freq();
				break;
	  case LPS:
				lps();
				break;
	  case RANDOM:
				rand_alg();
				break;
	}

	/*Stop timer*/
	ENDTIMING;

	/*Tell child timer process to wrapup this execution session*/
	sprintf(string,"q %ld %ld %ld",TIMINGS,info.x,info.y);
	write(writetimer,string,strlen(string));

	/*Stop capturing child's signals*/
	if(isbitclear(BATCHMODE)) {
	  notify_set_signal_func(frame[IMAGE],NOTIFY_FUNC_NULL,SIGUSR1,
				 NOTIFY_ASYNC);
	} else signal(SIGUSR1,SIG_DFL);

	/*Breakdown time into hours, minutes, and seconds*/
        info.time.hours = TIMINGS/3600;
        info.time.seconds = TIMINGS - (3600 * info.time.hours);
        info.time.minutes = info.time.seconds / 60;
        info.time.seconds -= (60 * info.time.minutes);

	/*Write out normalized correlation values*/
	if(alg != FREQ_DOMAIN) {
	  for(loop = 0;loop < totalvals;++loop)
	    fprintf(corr_resfp,"%lf\n",
		    (maxcorr - corrmatrix[loop]) / maxcorr * 100.0);
	} else {
		 /*Write out correlation numbers (which are being inverted and
		   normalized to match the pseudo-correlation numbers of the
		   other routines*/
	         for(loop = 0;loop < totalvals;loop += 2)
		  fprintf(corr_resfp,"%lf\n",corrmatrix[loop]/maxcorr * 100.0);
	       }
	free(corrmatrix);

	/*Send graphing instructions for this execution session to
	  graph file*/
	xvgr_legend(corr_resfp,dataset++,alg);
}

/*This is the function that gets called when the parent receives a signal
  from the child.  The current pattern position is sent back to the child
  (along with how many "clicks of time" have passed).

  Inputs:  none
  Outputs: none
  Locals:  string     - string message to send to child
  Globals: info       - structure containing results of the algorithm
	   sec        - counter of how many "clicks" of time have gone by
	   writetimer - file descriptor to write to child timer process
*/
void checkacc()
{
	register char string[40];

	sprintf(string,"%ld %ld %ld",sec++,info.x,info.y);
	write(writetimer,string,strlen(string));
}

