#include <stdio.h>
#include <fcntl.h>
#include <math.h>
#include <signal.h>
#include <sys/time.h>
#include "sherlock.h"
#include "timer.h"
#include "xvgr.h"
#include "error.h"

int dataset = 0;
int parent;
int running_alg;
long xcenter,ycenter;
FILE *dist_resfp;
struct itimerval timer_value;

/*Initialize the child timer process.

  Inputs:  none
  Outputs: none
  Locals:  loop           - loop through signals and file descriptors
	   pipe_fd        - interprocess file descriptors (for 2-way pipe)
  Globals: dist_resfp     - file pointer to distance graph data
	   dist_results   - name of file to save distance data to
	   errno          - number of last error encountered
	   parent         - process ID of parent
	   stdin	  - standard input
	   timer_value    - timer values for alarm to go off
	   writetimer     - file descriptor to write to child timer process
	   FATAL	  - error is fatal, program will terminate
	   FNDELAY	  - set up for non-blocking reads
	   F_SETFL        - set descriptor status flags
	   NSIG		  - number of signals
	   NULL		  - 0
	   SIGALRM	  - timer alarm signal
	   SIGINT	  - interrupt signal
	   SIG_DFL	  - set up default signal action
	   TIMER_INTERVAL - number of seconds for timer setting
*/
init_timer()
{
	register int pipe_fd[2],loop;
	void timer_alarm(),timer_quit();

	/*Set up for 2-way pipe for interprocess communication*/
	if(pipe(pipe_fd) == -1) error(FATAL,errno);

	switch((child = fork())) {
	  case 0:				/*Child*/
			/*Get parent PID*/
			parent = getppid();

			/*Redirect stdin to pipe file descriptor*/
			dup2(pipe_fd[0],fileno(stdin));

                   	/*Close unneeded file descriptors*/
                   	for(loop = getdtablesize();loop > 2;--loop)
                     	  close(loop);

                   	/*Ignore unneeded signals*/
                   	for(loop = 0;loop < NSIG;++loop)
                     	  signal(loop,SIG_DFL);

			/*Set up for SIGALRM & SIGINT*/
			signal(SIGALRM,timer_alarm);
			signal(SIGINT,timer_quit);

			/*Set up timer values*/
			timer_value.it_interval.tv_sec = TIMER_INTERVAL;
			timer_value.it_interval.tv_usec = 0;
			timer_value.it_value.tv_usec = 0;

			/*Open results file*/
			if((dist_resfp = fopen(dist_results,"w")) == NULL)
			 error(FATAL,errno);

			set_y_viewport(dist_resfp);

			/*Idle timer until it's needed*/
			timer_wait();

			exit(0);		/*Should never get here*/
	  case -1:				/*Error*/
			error(FATAL,errno);
	  default:				/*Parent*/
			close(pipe_fd[0]);
			writetimer = pipe_fd[1];
			fcntl(writetimer,F_SETFL,FNDELAY);
			break;
	}
}

/*Sit in idle mode until the parent process sends a message starting the
  timer cycle.

  Inputs:  none
  Outputs: none
  Locals:  string         - string read in from parent
  Globals: dataset        - number of the dataset this session will create
	   dist_resfp     - file pointer to distance graph data
	   running_alg    - algorithm that's running
	   stdin          - standard input
	   timer_value    - timer values for alarm to go off
	   xcenter        - X coordinate center of pattern in image
	   ycenter        - Y coordinate center of pattern in image
	   ITIMER_REAL    - set alarm clock in real time
	   NULL           - 0
	   TIMER_INTERVAL - number of seconds before alarm clock signals
*/
timer_wait()
{
	register char string[20];

	/*Contine forever, or until a SIGINT is received, whichever comes
	  first*/
	while(1) {
	  /*Block reading from parent until there is something to read
	    (which means to start up timer process)*/
	  read(fileno(stdin),string,19);

	  /*Verify that message means to start (with the letter 's')*/
	  if(string[0] == 's') {
	    /*Set up alarm clock*/
	    timer_value.it_value.tv_sec = TIMER_INTERVAL;
	    setitimer(ITIMER_REAL,&timer_value,NULL);

	    /*Rest of message contains the algorithm that is running (needed
	      for graph commands) and the coordinates of the center of the
	      pattern within the image (needed for calculating distance)*/
	    sscanf(&string[2],"%d %ld %ld",&running_alg,&xcenter,&ycenter);

	    /*Write first distance to graph file*/
	    fprintf(dist_resfp,"0 %lf\n",pow(pow((double) xcenter,2.0) +
		    pow((double) ycenter,2.0),0.5));

	    /*Start alarm clock timer process*/
	    timer();
	  }

	  ++dataset;
	}
}

/*As the alarm clock runs in the "background", it will send signals to the
  parent at which time the parent will send messages to the child.  This
  funtion will interperet those messages until a "quit" message is given, at
  which point, the child goes back to an idle state.  Parse each message
  that arrives calculating distance and saving that value to the distance
  graph file.  When we receive the "quit" message, put graphing commands
  into the data file for the algorithm that finished executing.

  Inputs:  none
  Outputs: none
  Locals:  quit           - flag indicating when we received the "quit" message
	   string         - message from parent
  Globals: dataset        - number of the dataset this session created
	   dist_resfp     - file pointer to distance graph data
	   running_alg    - algorithm that's running (or has just finished)
	   stdin          - standard input
	   timer_value    - timer values for alarm to go off
	   ITIMER_REAL    - set alarm clock in real time
	   NULL           - 0
	   TIMER_INTERVAL - number of seconds before alarm clock signals
*/
timer()
{
	register int quit = 0;
	register char string[20];

	/*Continue until we get the "quit" message*/
	while(!quit) {
	  /*Zero out string, all characters may NOT get overwritten*/
	  bzero(string,20);

	  read(fileno(stdin),string,19);
	  if(string[0] == 'q') {
	    /*If quitting, set alarm clock to 0 to stop it from "ringing"*/
	    timer_value.it_value.tv_sec = 0;
	    setitimer(ITIMER_REAL,&timer_value,NULL);

	    /*Parse final values*/
	    parse_string(&string[2],1);

	    /*Write graph legend commands for this algorithm*/
	    xvgr_legend(dist_resfp,dataset,running_alg);

	    quit = 1;
	  } else parse_string(string,TIMER_INTERVAL);
	}
}

/*Parse message string from parent and caculate and write out the distance
  from where the parent thinks the pattern is located to where it actually
  is.

  Inputs:  multiplier - number of seconds in a "click of time"
	   string     - message from parent
  Outputs: none
  Locals:  numclicks  - number of "clicks of time" that has passed so far
	   xpos       - X coordinate of where parent thinks pattern is
	   ypos       - Y coordinate of where parent thinks pattern is
  Globals: dist_resfp - file pointer to distance graph data
	   xcenter    - X coordinate center of pattern in image
	   ycenter    - Y coordinate center of pattern in image
*/
parse_string(string,multiplier)
register int multiplier;
register char *string;
{
	long numclicks,xpos,ypos;

	sscanf(string,"%ld %ld %ld",&numclicks,&xpos,&ypos);

	fprintf(dist_resfp,"%ld [%d,%d] %lf\n",numclicks * multiplier,
		xpos,ypos,
		pow(pow((double) (xpos - xcenter),2.0) +
		pow((double) (ypos - ycenter),2.0),0.5));
}


/*This function is called when the alarm clock "rings".  Send a signal to
  the parent process to get it to send a message back to the child about
  where it thinks the pattern is at this moment in time.

  Inputs:  none
  Outputs: none
  Locals:  none
  Globals: parent  - process ID of parent
	   SIGUSR1 - user defined signal #1
*/
void timer_alarm()
{
	kill(parent,SIGUSR1);
}

/*Write the final graph commands to the distance graph file before exiting
  permanently.

  Inputs:  none
  Outputs: none
  Locals:  none
  Globals: dist_resfp - file pointer to distance graph data
*/
void timer_quit()
{
	xvgr_commands(dist_resfp,"Seconds","Distance");

	fclose(dist_resfp);

	exit(0);
}

